// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTACommManager.java,v 1.10 2001/06/21 14:57:09 veit Exp $

package jfta.base;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.system.*;

public abstract class FTACommManager {

    // adjust this constant to increase number of clients
    public final static int MAX_CLIENTS = 20;

    protected final static FTAService _svc = FTAService.Instance();

    protected FTAServerChannel _srvchan;
    protected FTAListenerList _srvlisteners;
    protected FTAClientChannel[] _connections;
    protected FTAListenerI _listener;
    protected String _myhost;
    protected FTASysSub _mysyssub;
    protected ThreadGroup _mytg;
    protected boolean _isHost0;

    public FTACommManager(FTASysSub syssub,int hostnum) {
	_mysyssub = syssub;
	_isHost0 = (hostnum==0);
	_connections = new FTAClientChannel[MAX_CLIENTS];
	_myhost = _svc.getFQDNname();
	_mytg = syssub.getSysThreadGroup();

	// YOU MUST INITIALIZE listeners in subclass
	// _listener = new CommListener();
	// _srvlisteners = _srvchan.getListeners();
    }

    public void uninstall() {
	_listener.uninstall(this);
	for (int i= 0; i<MAX_CLIENTS; i++)
	    closeClient(i);
    }

    /**
     * distribute connection information
     */
    public void distributeHostInfo() { /*do nothing here*/ }

    /**
     * get the server port
     */
    public final int getServerPort() {
	return _srvchan.getPort();
    }

    /**
     * add a channel to send data to a specified host
     */
    public abstract void openClient(int hostnum,String host,int srvport);

    /**
     * remove a channel for sending data
     */
    public final void closeClient(int hostnum) {
	try {
	    FTAClientChannel cl = _connections[hostnum];
	    _connections[hostnum] = null;
	    if (cl != null) cl.close();
	}
	catch (IOException e) {}
    }

    /**
     * add a listener to this server channel
     */
    public final void addListener(int msg,FTAListenerI listener) {
	addListener(msg,listener,-1);
    }

    /**
     * add a listener to this server channel with a filter 
     * for certain originators
     */
    public final void addListener(int msg,FTAListenerI listener,
				  int filter) {
	//System.out.println("Register "+FTAMsg.getTypeNum(msg)+
	// " filter="+filter);
	_srvchan.addListener(msg,listener,filter);
    }
    
    /**
     * remove a listener
     */
    public final void removeListener(int msg,FTAListenerI listener) {
	_srvchan.removeListener(msg,listener);
    }

    /**
     * send a message to a host
     */
    public /*synchronized*/ void sendMsg(int hostnum,FTAMsg msg) {
	FTAClientChannel cl = _connections[hostnum];
	if (cl != null)
	    cl.sendMsg(msg);
	else
	    FTADebug.fatal("Failed to send message to "+hostnum);
	//System.out.println("send "+msg+" done");
	msg.discard(); // discard send
    }

    /**
     * broadcast a message to all hosts
     */
    public final /*synchronized*/ void broadcastMsg(FTAMsg msg) {
	for (int i= 0; i< MAX_CLIENTS; i++) {
	    FTAClientChannel cl = _connections[i];
	    if (cl != null) {
		//FTADebug.log(i,0,"broadcast "+msg);
		cl.sendMsg(msg);
	    }
	}
	//System.out.println("broadcast "+msg+" done");
	msg.discard(); // discard broadcast
    }

    public class CommListener extends FTAListenerAdapter {
	public CommListener() {
	    addListener(FTAMsg.CREATEHOSTDONE,this);
	    addListener(FTAMsg.REMOVEHOST,this);
	    addListener(FTAMsg.ADDHOST,this);
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.CREATEHOSTDONE,this);
	    comm.removeListener(FTAMsg.REMOVEHOST,this);
	    comm.removeListener(FTAMsg.ADDHOST,this);
	}

	public synchronized void notifyCreateHostDone(int hostnum,
					 String hostname,
					 int port) {
	    if (hostnum != 0)
		openClient(hostnum,hostname,port);
	}

	public synchronized void notifyRemoveHost(int hostnum) {
	    closeClient(hostnum);
	}

	public synchronized void notifyAddHost(int hostnum,
				  String hostname,int port) {
	    if (hostnum != 0 && !_myhost.equals(hostname))
		openClient(hostnum,hostname,port);
	}
    }

    // bypass message passing
    public final synchronized void directFlipOrder(int ennum,int tbnum) {
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.FLIPORDER);
	while (elem != null) {
	    if (elem.matches(tbnum))
		elem.listener.notifyFlipOrder(ennum);
	    elem = elem.next;
	}
    }

    public final synchronized void directCycle(int ennum,int tbnum) {
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.CYCLE);
	while (elem != null) {
	    if (elem.matches(tbnum))
		elem.listener.notifyCycle(ennum);
	    elem = elem.next;
	}
    }

    public final synchronized void directConsent(int ennum,int tbnum) {
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.CONSENT);
	while (elem != null) {
	    if (elem.matches(tbnum))
		elem.listener.notifyConsent(ennum);
	    elem = elem.next;
	}
    }

    public final synchronized void directPostFlip(int tbnum,int ennum) {
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.POSTFLIP);
	while (elem != null) {
	    if (elem.matches(ennum))
		elem.listener.notifyPostFlip(tbnum);
	    elem = elem.next;
	}
    }

    public final synchronized void directPreFlip(int tbnum,int ennum) {
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.PREFLIP);
	while (elem != null) {
	    if (elem.matches(ennum))
		elem.listener.notifyPreFlip(tbnum);
	    elem = elem.next;
	}
    }

    public final synchronized void directStartCycle(int objnum,long cycletime,long millis) {
	// special case local host
	FTAListenerItem elem = _srvlisteners.first(FTAMsg.STARTCYCLE);
	while (elem != null) {
	    if (elem.matches(objnum))
		elem.listener.notifyStartCycle(cycletime,millis);
	    elem = elem.next;
	}
	
	// other hosts
	for (int i= 1; i< MAX_CLIENTS; i++) {
	    FTAClientChannel cl = _connections[i];
	    if (cl != null) {
		DataOutputStream out = cl.getStream();
		try {
		    // build a WorkNow/FlipNow message packet.
		    out.writeInt(FTAMsg.FTAMAGIC);
		    out.writeShort((short)FTAMsg.STARTCYCLE);
		    out.writeInt(objnum);
		    out.writeLong(cycletime);
		    out.writeLong(millis);
		}
		catch(IOException e) {
		    e.printStackTrace();
		}
	    }
	}
    }

    public final synchronized void directAckCycle(int hostnum,int objnum,
						  long cycletime,boolean fin) {
	// special case local host
	if (_isHost0) {
	    FTAListenerItem elem = _srvlisteners.first(FTAMsg.ACKCYCLE);
	    while (elem != null) {
		if (elem.matches(objnum))
		    elem.listener.notifyAckCycle(0,cycletime,fin);
		elem = elem.next;
	    }
	} else {
	    FTAClientChannel cl = _connections[0];
	    if (cl != null) {
		DataOutputStream out = cl.getStream();
		try {
		    // build a Workdone/FlipDone message packet.
		    out.writeInt(FTAMsg.FTAMAGIC);
		    out.writeShort((short)FTAMsg.ACKCYCLE);
		    out.writeInt(hostnum);
		    out.writeInt(objnum);
		    out.writeLong(cycletime);
		    out.writeBoolean(fin);
		}
		catch(IOException e) {
		    e.printStackTrace();
		}
	    }
	}
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
