// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAIHash.java,v 1.4 2001/06/21 14:57:09 veit Exp $

package jfta.base;

import java.util.*;

/**
 * this class implements a Hashtable with an int as key and
 * an object as value
 */

public class FTAIHash {

    IEntry[] _table;
    int _tsize;
    int _sz;

    /**
     * construct a hash table
     */
    public FTAIHash() {
	this(37);
    }

    /**
     * construct a hash table with a given table size. The table 
     * size should be a prime number.
     */
    public FTAIHash(int tblsize) {
	_tsize = tblsize;
	_table = new IEntry[_tsize];
	_sz = 0;
    }

    public synchronized void clear() {
	for (int i=0; i<_tsize; i++) {
	    IEntry e = _table[i];
	    while (e != null) {
		e._value = null;
		IEntry e2 = e._next;
		e._next = null;
		e2 = e;
	    }
	    _table[i] = null;
	}
    }

    public synchronized void put(int key,Object value) {
	int hash = (key & 0x7fffffff) % _tsize;
	_table[hash] = new IEntry(_table[hash],key,value);
	_sz++;
    }

    public boolean isEmpty() {
	return _sz == 0;
    }

    public int size() {
	return _sz;
    }

    public synchronized Object remove(int key) {
	int hash = (key & 0x7fffffff) % _tsize;
	IEntry e = _table[hash];
	if (e==null) return null;
	else if (e._key==key) {
	    _table[hash] = e._next;
	    _sz--;
	} else {
	    IEntry eprev = e;
	    e = e._next;
	    while (e != null) {
		if (e._key==key) {
		    eprev._next = e._next;
		    _sz--;
		    break;
		}
	    }
	    if (e==null)
		return null;
	}

	Object o = e._value;
	e._value = null;
	e._next = null;
	return o;
    }

    public synchronized Object get(int key) {
	int hash = (key & 0x7fffffff) % _tsize;
	IEntry e = _table[hash];
	while (e != null) {
	    if (e._key == key)
		return e._value;
	    e = e._next;
	}
	return null;
    }

    public synchronized boolean containsKey(int key) {
	int hash = (key & 0x7fffffff) % _tsize;
	IEntry e = _table[hash];
	while (e != null) {
	    if (e._key == key)
		return true;
	    e = e._next;
	}
	return false;
    }

    public synchronized boolean containsValue(Object val) {
	for (int i=0; i<_tsize; i++) {
	    IEntry e = _table[i];
	    while (e != null) {
		if (e._value.equals(val))
		    return true;
		e = e._next;
	    }
	}
	return false;
    }

    public Enumeration elements() {
	return new IEnum(_table,_tsize);
    }
}

class IEnum implements Enumeration {
    private int n;
    private IEntry e;
    private IEntry[] table;
    private int tsize;
    IEnum(IEntry[] t,int ts) {
	table = t;
	tsize = ts;
	for (n = 0; n<tsize; n++) {
	    if (table[n] != null) {
		e = table[n];
		break;
	    }
	}
    }

    public boolean hasMoreElements() {
	return e != null;
    }

    public Object nextElement() {
	if (e == null)
	    return null;
	else {
	    Object o = e._value;
	    e = e._next;
	    if (e == null) {
		for (n++; n<tsize; n++) {
		    if (table[n] != null) {
			e = table[n];
			break;
		    }
		}
	    }
	    return o;
	}
    }
}

class IEntry {
    int _key;
    Object _value;
    IEntry _next;
    
    IEntry(IEntry e,int key,Object value) {
	_next = e;
	_key = key;
	_value = value;
    }
}



/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
