// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAOfficeManager.java,v 1.16 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.util.*;
import jfta.system.*;

public class FTAOfficeManager extends FTAListenerAdapter {

    // recompile system if you need more
    private final static int MAX_OFFICES = 20;

    final static int WAITFORACK = 2;
    final static int GOTACK = 3;

    private int _waitforack;
    private FTAOfficeRef[] _offices;
    private int _ocount;
    //private FTAIHash _offices;
    private FTASysSub _mysyssub;
    private FTACommManager _comm;
    private int _myobjnum;
    private long _cycletime;
    private long _cyclemillis;

    public FTAOfficeManager(FTASysSub mysyssub,int objnum) {
	//_offices = new FTAIHash();
	_offices = new FTAOfficeRef[MAX_OFFICES];
	_ocount = 0;
	_mysyssub = mysyssub;
	_comm = mysyssub.getCommManager();
	_myobjnum = objnum;
	_cycletime = 0;
	_cyclemillis = 0;

	_comm.addListener(FTAMsg.REMOVEHOST,this);
    }

    public void uninstall() {
	_comm.removeListener(FTAMsg.REMOVEHOST,this);
    }
    
    public FTAOfficeRef[] getOffices() {
	return _offices;
    }

    //public Enumeration elements() {
    //return _offices.elements();
    //}

    /**
     * handle a create{TB,EN}Done message
     */
    public boolean createDone(int hostnum) {
	synchronized(_offices) {
	    String hostname = _mysyssub.getNameById(hostnum);
	    //_offices.put(hostnum,new FTAOfficeRef(hostname));
	    if (_offices[hostnum]==null)
		_ocount++;
	    _offices[hostnum] = new FTAOfficeRef(hostname);
	    return _ocount==1;
	}
    }

    /**
     * handle a removeHost message
     * not yet implemented, so it will result in 'unimplemented'
     * message from FTAListenerAdapter
     */
    public void notifyRemoveHost(int hostnum) {
	//_offices.remove(hostnum);
	_offices[hostnum] = null;
	_ocount--;
	//FTADebug.fatal("Officemanager: removeHost not implemented");
    }

    /**
     * start {work,flip} phase
     * @param fliptick: true for flip, false for tick
     */
    public void startCycle(boolean fliptick) {
	// process new offices
	synchronized(_offices) {
	    _waitforack = 0;
	    for (int i=0; i<MAX_OFFICES; i++) {
		FTAOfficeRef ref = _offices[i];
		if (ref != null) {
		    ref.changeState(WAITFORACK);
		    _waitforack++;
		}
	    }
	}

	_cycletime++;
	_cyclemillis = System.currentTimeMillis();
	_comm.directStartCycle(_myobjnum,_cycletime,_cyclemillis);
    }

    /**
     * return current cycletime
     */
    public long getCycleTime() {
	return _cycletime;
    }

    /**
     * handle a {work,flip}Done message
     */
    public boolean acknowledgeDone(int hostnum,long cyc,boolean fin) {
        //FTADebug.log(0,_myobjnum,"ack for "+cyc);
	FTAOfficeRef ref = _offices[hostnum];
	if (ref != null) {
	    _waitforack--;
	    //int is = ref.changeState(GOTACK);
	    //if (is != WAITFORACK)
	    //FTADebug.error(0,_myobjnum," Invalid Ack: not waiting: state="+is);

	    ref.changeState(GOTACK);
	    if (fin)
		notifyRemoveHost(hostnum);
	} else {
	    _waitforack--;
	    FTADebug.error(0,_myobjnum,
			   " OM: UNKNOWN ACK "+hostnum);
	}
	
	// true if all have replied
	return _waitforack==0;
    }

    public boolean hasOffices() {
	return _ocount > 0;
    }
    
    public String toString() {
	StringBuffer sb = new StringBuffer();
	for (int i=0; i<_offices.length; i++) {
	    FTAOfficeRef ref = _offices[i];
	    if (ref != null)
		sb.append(ref+" ");
	}
	//Enumeration it = _offices.elements();
	//while (it.hasMoreElements()) {
	//sb.append(it.nextElement().toString()+' ');
	//}
	return sb.toString();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
