// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTADDaemonServlet.java,v 1.4 2001/06/21 14:57:10 veit Exp $

package jfta.bootstrap;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.exception.*;
import jfta.http.*;

public class FTADDaemonServlet extends FTAServlet {

    private static FTAService _svc = FTAService.Instance();
    private int _updatelatency = 5;
    private FTADaemon _daemon;
    private FTAPropertyFile _props;

    /**
     * mandatory constructor
     */
    public FTADDaemonServlet() {
	super();
    }

    /**
     * init method
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {

	// get daemon
	_daemon = (FTADaemon)server.getContainer();

	// get property
	_updatelatency = _daemon.getLatency();
    }

    /**
     * reply method.
     */
    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();
	String tail = req.getTail();
	long now = System.currentTimeMillis();
	Date dnow = new Date(now);
	/* handling for cgi-bin */
	try {
	    if (head.equals("/finddaemons.ftad")) {
		findDaemons(now);
		
		// fall through
		tail = "/";
	    } else if (head.equals("/listdaemons.ftad"))
		return listDaemons();
	    else if (head.equals("/shutdownalldaemons.ftad"))
		return doShutall();
	    else if (head.equals("/shutdowndaemon.ftad"))
		return doShutdown();

	    if (tail != null) {
		if (tail.equals("/") || tail.equals("/index.html"))
		    return doIndex(dnow);
		else if (tail.equals("/newsys.html"))
		    return doNewsys();
		else if (tail.equals("/shutsys.html"))
		    return doShutsys();
	    }
	    // nothing matched
	    return FTAErrorReply.error404();
	}
	catch(Exception e) {
	    return FTAErrorReply.error500(e.toString());
	}
    }

    private FTAServletReply doShutdown() {
	Thread thr = new Thread() {
		public void run() {
		    _svc.delay(3000);
		    System.exit(0);
		}
	    };
	thr.run();
	return _daemon.replyText("Shutdown done");
    }

    private FTAServletReply doShutall() {
	Thread thr = new Thread() {
		public void run() {
		    _svc.delay(3000);
		    _daemon.sendMsg(FTAShutdownMsg.get());
		}
	    };
	thr.run();
	return _daemon.replyText("Shutdown sent");
    }

    private FTAServletReply doNewsys() {
	if (_props == null) _props = _daemon.getInitActors();
	FTAHtmlFile f = new FTAHtmlFile("Start new FTASystem");
	f.text("To start a new FTA System task on ");
	f.b("this"); f.text(" host, fill in the following form:");
	f.beginForm("/cgi-bin/newsystem.ftad");
	f.submitButton("Start");
	f.resetButton("Clear Form");
	f.beginTable();
	f.tableNewRow("System name:"); f.tableColumn();
	f.inputField(20,"sysname","");
	f.tableNewRow("Initial actor:"); f.tableColumn();
	_props.selectBox(f,"actor","--select or add new-->",
			 "--select or add new-->");
	f.tableColumn(); f.inputField(20,"newactor","");
	f.tableNewRow("Argument(s):"); f.tableColumn();
	f.inputField(20,"arg0","");
	f.tableNewRow(""); f.tableColumn();
	f.inputField(20,"arg1","");
	f.tableNewRow(""); f.tableColumn();
	f.inputField(20,"arg2","");
	f.tableNewRow(""); f.tableColumn();
	f.inputField(20,"arg3","");
	f.tableNewRow(""); f.tableColumn();
	f.inputField(20,"arg4","");
	f.endTable();
	f.endForm();
	return _daemon.stdFinish(f);
    }

    private FTAServletReply doShutsys() {
	return FTAErrorReply.error500("Shutdown System: not implemented yet");
    }

    private FTAServletReply doIndex(Date dnow) {
	Hashtable daemons = _daemon.getDaemons();
	FTAHtmlFile f = new FTAHtmlFile("Index of FTADaemons");
	f.h2("Known FTA Daemons at port " + 
	     _svc.FTADAEMONPORT);
	_daemon.addFormButton(f,"Update Daemon List",
			      "/cgi-bin/finddaemons.ftad");
	f.p("("+_updatelatency+" second(s) latency)");
	f.p("Current time is " + dnow);
	_daemon.addRemoteLinkList(f,daemons,"/index.html","");
	return _daemon.stdFinish(f);
    }

    private FTAServletReply listDaemons() {
	return _daemon.listall(_daemon.getDaemons());
    }

    private void findDaemons(long now) 
	throws InterruptedException {
	_daemon.sendMsg(FTADaemonReqMsg.get(now));
	Thread.sleep(_updatelatency*1000);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
