// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTADSystemServlet.java,v 1.6 2001/06/21 14:57:10 veit Exp $

package jfta.bootstrap;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.exception.*;
import jfta.http.*;
import jfta.system.*;

public class FTADSystemServlet extends FTAServlet {

    private static FTAService _svc = FTAService.Instance();
    private int _updatelatency = 5;
    private FTADaemon _daemon;

    /**
     * mandatory constructor
     */
    public FTADSystemServlet() {
	super();
    }

    /**
     * init method
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {

	// get properties
	_daemon = (FTADaemon)server.getContainer();
	_updatelatency = _daemon.getLatency();
    }

    /**
     * reply method.
     */
    public FTAServletReply reply(FTAServletRequest req) {

	String head = req.getHead();
	String tail = req.getTail();
	Hashtable args = req.getArgs();
	long now = System.currentTimeMillis();
	Date dnow = new Date(now);
	/* handling for cgi-bin */
	try {
	    if (head.equals("/findsystems.ftad")) {
		findSystems(now);

		// fall through
		head = "/systems";
		tail = "/";
		req.modifyHeadTail(head,tail);
	    } else if (head.equals("/newsystem.ftad")) {
		return newSystem(args);
	    } else if (head.equals("/removesystem.ftad")) {
		return removeSystem(args);
	    } else if (head.equals("/controlsystem.ftad")) {
		return controlSystem(args);
	    } else if (head.equals("/listsystems.ftad")) {
		return listSystems();
	    }

	    if (head.equals("/systems")) {
		// split again into head and tail
		head = req.shift();
		tail = req.getTail();

		if (head.equals("/") || head.equals("/index.html"))
		    return doIndex(dnow);
	    } else if (head.equals("/fta")) {
		// split again into head and tail
		head = req.shift();
		tail = req.getTail();
		Hashtable systems = _daemon.getLocalSystems();
		FTAServletI sm = 
		    (FTAServletI)systems.get(head.substring(1));
		if (sm != null) {
		    req.shift();
		    return sm.reply(req);
		}
	    }

	    // catchall
	    return FTAErrorReply.error404();
	}
	catch(Exception e) {
	    return new FTAErrorReply(500,e);
	}
    }

    private FTAServletReply doIndex(Date dnow) {
	Hashtable remote = _daemon.getRemoteSystems();
	FTAHtmlFile f = new FTAHtmlFile("Index of FTA Systems");
	f.beginForm("/cgi-bin/findsystems.ftad");
	f.submitButton("Update System list");
	f.endForm();
	f.p("("+_updatelatency+" seconds latency)");
	f.p("Current time is " + dnow);
	f.h2("Known FTA Systems");
	_daemon.addRemoteLinkList(f,remote,"/fta/","/");
	return _daemon.stdFinish(f);
    }

    private FTAServletReply newSystem(Hashtable args) {
	String sysname = (String)args.remove("sysname");
	String acname = (String)args.remove("actor");
	if (acname.equals("--select or add new-->")) {
	    acname = (String)args.remove("newactor");
	    FTAPropertyFile props = _daemon.getInitActors();
	    if (acname.equals(""))
		acname = null;
	    else
		props.add(acname,acname);
	}
	String[] arglist = getArglist(args);

	if (!sysname.equals("") && acname != null) {
	    // acknowledge creation even if the actual creation will fail
	    // to avoid deadlocks in daemon
	    _daemon.createSystem(sysname,acname,arglist);
	    _svc.delay(2000);
	} else
	    _daemon.logMessage(FTADaemon.LOG_ERROR,
			       "Creation failed: invalid or missing sysname/actorname passed to FTADaemon");
	return _daemon.redirect("/serverlog.html");
    }

    /**
     * convert a list "argNN=value" to a String array
     */
    private String[] getArglist(Hashtable args) {
	Vector argv = new Vector();
	for (int i=0; ; i++) {
	    String key = "arg"+i;
	    if (!args.containsKey(key)) break;
	    String val = (String)args.get(key);
	    if (val.equals("")) break;
	    argv.add(args.get(key));
	}
	int sz = argv.size();
	String[] result = new String[sz];
	for (int n=0; n<sz; n++) {
	    result[n] = (String)argv.elementAt(n);
	}
	return result;
    }

    private FTAServletReply removeSystem(Hashtable args) {
	String sysname = (String)args.get("sysname");
	if (sysname != null && _daemon.removeSystem(sysname))
	    return _daemon.replyDone();
	else
	    return _daemon.replyFailed("SS002");
    }

    private FTAServletReply listSystems() {
	return _daemon.listall(_daemon.getLocalSystems());
    }

    private FTAServletReply controlSystem(Hashtable args) 
	throws Exception {
	String sysname = (String)args.get("sysname");
	String fc = (String)args.get("function");
	String cnt = (String)args.get("code");
	int function = _svc.atoi(fc,-1);
	int count = _svc.atoi(cnt,0);

	if (sysname != null && function != -1) {
	    //System.out.println("sysname="+sysname+" func="+function);
	    _daemon.controlSystem(sysname,function,count);
	} else
	    _daemon.logMessage(FTADaemon.LOG_ERROR,
			       "Controlsystem failed: invalid or missing sysname/functioncode passed to FTADaemon");
	switch (function) {
	case FTASysSub.FTACTLSTART:
	    return _daemon.redirect("/fta/"+sysname+"/$LOG$/");
	case FTASysSub.FTACTLSHUTDOWN:
	    return _daemon.redirect("/serverlog.html");
	case FTASysSub.FTACTLINITIALIZING:
	case FTASysSub.FTACTLSTOP:
	default:
	    return _daemon.redirect("/fta/"+sysname+"/");
	}
    }

    private void findSystems(long now) 
	throws InterruptedException {
	_daemon.sendMsg(FTASystemReqMsg.get(now));
	_svc.delay(_updatelatency*1000);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
