// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: Client.java,v 1.3 2001/06/21 14:57:13 veit Exp $

package jfta.demo.cs3;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

/**
 * This client looks at the tagboard for an reply for the own id, and
 * will display received reply, and issue a new request to the server.
 * The request contains the own id, a dummy request number and 
 * an operation (+,-,*,/,Q, where Q is an ill-formed request)
 * and two arguments.
 *
 * @author Holger Veit
 * @version $Id: Client.java,v 1.3 2001/06/21 14:57:13 veit Exp $
 */
public class Client extends FTAActor implements FTAActorI {
    
    private int myid;
    private int reqid;
    
    public static char[] opers = { '+', '-', '*', '/', 'Q' };
    private FTACallback cb;
    
    /**
     * callback helper class
     */
    private class ReplyCallback extends FTATBCallback {
	public ReplyCallback(FTAActor ac,String tb) {
	    super(ac,tb);
	}

	public int eval2() throws FTAEvalActorException {
	    FTATagIterator iter = getTagIterator(tb,Server.Reply3Tag);
	    while (iter.hasNext()) {
		Reply3Tag tag = (Reply3Tag)iter.next();
		if (tag.isClient(myid)) {
		    int id = tag.getRequestId();
		    if (tag.isOkay()) {
			userMessage(LOG_STD,
				    "Client("+myid+"): req#=" + id +
				    ", result="+tag.getResult());
		    } else {
			userMessage(LOG_ERROR,
				    "Client("+myid+"): req#=" + id +
				    ", rejected by server");
		    }
		}
	    }
	    return KEEPALIVE;
	}
    }
    
    /**
     * mandatory eval method
     * @return FTAActor.TERMINATE if the actor terminates after this
     *   routine has been called, FTAActor.KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {
	/*
	 * check for activity on server tagboard
	 */
	cb.eval();

	/*
	 * generate a new request, but at most 10 (restrict run)
	 */
	if (reqid <= 10) {
	    int rid = reqid + 100*myid;
	    int a1 = (int)(Math.random()*10.0);
	    int a2 = (int)(Math.random()*10.0);
	    int op = (int)(Math.random()*5.0);
	    putTag(Server.ReqTB,new Req3Tag(myid,rid,a1,a2,op));
	    flipOrder(Server.ReqTB);
	    userMessage(LOG_STD,"new req# = " + rid +
			", Operation = " + a1 + opers[op] + a2);
	    
	    // increment request id
	    reqid++;
	    
	}
	return KEEPALIVE;
    }
    
    /**
     * mandatory init method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incoreect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	if (args.length < 1)
	    throw new FTAArgumentException("Expected 1 argument");
	try {
	    myid = Integer.parseInt(args[0]);
	} 
	catch(NumberFormatException e) {}
	
	reqid = 1;
    }
    
    /**
     * mandatory empty constructor
     */
    public Client() {
	super();
	cb = new ReplyCallback(this,Server.ReplyTB);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
