// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: Reply3Tag.java,v 1.2 2001/06/21 14:57:13 veit Exp $

package jfta.demo.cs3;

import jfta.exception.*;
import jfta.tags.*;
import jfta.http.*;

/**
 * This tag stores a reply for the sample client/server system.
 * It is an example of a tag to be used by both tag methods: it
 * implements a custom constructor as well as the older init() method.
 * @author Holger Veit
 * @version $Id: Reply3Tag.java,v 1.2 2001/06/21 14:57:13 veit Exp $
 */
public class Reply3Tag extends FTATag {
    
    /**
     * attributes
     */
    private int clid;
    private int reqid;
    private int result;
    private boolean ok;
    
    /**
     * access methods
     */
    public int getClientId() {
	return clid;
    }
    public int getRequestId() {
	return reqid;
    }
    public int getResult() {
	return result;
    }
    public boolean isOkay() {
	return ok;
    }
    
    public boolean isClient(int myid) {
	return myid == clid;
    }
    
    /**
     * return estim. number of tags
     * @return the approx. number of tags to be expected during a flip cycle
     */
    public int getCapacityHint() {
	return 50;
    }
    
    /**
     * empty constructor
     */
    public Reply3Tag() {
	super();
    }
    
    /**
     * custom constructor
     */
    public Reply3Tag(int cid,int rid,int res,boolean okay) {
	super();
	clid = cid;
	reqid = rid;
	result = res;
	ok = okay;
    }
    
    /**
     * @param args the argument string used to initialize the tag
     * @exception FTAArgumentException Thrown in case of an invalid or missing
     *   argument
     */
    public void init(String[] args) throws FTAArgumentException {
	if (args.length < 4)
	    throw new FTAArgumentException("Expected 4 args");
	try {
	    clid = Integer.parseInt(args[0]);
	    reqid = Integer.parseInt(args[1]);
	    result = Integer.parseInt(args[2]);
	    ok = args[3].equals("true");
	}
	catch(NumberFormatException e) {
	    throw new FTAArgumentException("Invalid parameter");
	}
    }


    /**
     * provide HTML Information about this object and its 
     * contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHtmlFile f = new FTAHtmlFile("Tag info for ReqTag");
	f.beginUL();
	f.propitem("Hashcode",hashCode()+"");
	f.propitem("Client Id",clid+"");
	f.propitem("Request Id",clid+"");
	f.propitem("Result",result+"");
	f.propitem("Result okay",ok+"");
	f.finish();
	return f;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
