// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: Server.java,v 1.3 2001/06/21 14:57:13 veit Exp $

package jfta.demo.cs3;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

/**
 * This actor just prints a message each time it is called
 * @author Holger Veit
 * @version $Id: Server.java,v 1.3 2001/06/21 14:57:13 veit Exp $
 */
public class Server extends FTAActor implements FTAActorI {
    
    public final static String ReqTB = "request";
    public final static String ReplyTB = "reply";
    public final static String Req3Tag = "jfta.demo.cs3.Req3Tag";
    public final static String Reply3Tag = "jfta.demo.cs3.Reply3Tag";

    private long lastSeen = 0;
    
    /**
     * mandatory eval method.
     * @return FTAActor.TERMINATE if the actor terminates after this
     *   routine has been called, FTAActor.KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {
	try {
	    // This is the old way to verify a tagboard change
	    // see Client.java to find out about the Callback-class based approach
	    long now = getTagboardTime(ReqTB,RELATIVE);
	    if (now <= lastSeen)
		return KEEPALIVE;

	    lastSeen = now;

	    int i = 0;
	    FTATagIterator iter = getTagIterator(ReqTB,Req3Tag);
	    while (iter.hasNext()) {
		Req3Tag tag = (Req3Tag)iter.next();
		int clid = tag.getClientId();
		int rid = tag.getRequestId();
		int n1 = tag.getArg1();
		int n2 = tag.getArg2();
		int oper = tag.getOper();

		int result = 0;
		boolean ok = true;
		try {
		    switch (oper) {
		    case 0:
			result = n1 + n2;
			break;
		    case 1:
			result = n1 - n2;
			break;
		    case 2:
			result = n1 * n2;
			break;
		    case 3:
			result = n1 / n2;
			break;
		    default:
			ok = false;
		    }
		}
		catch(ArithmeticException e) {
		    ok = false;
		}

		putTag(ReplyTB, new Reply3Tag(clid,rid,result,ok));
		flipOrder(ReplyTB);
		userMessage(LOG_STD,
			    "Server: req# =" + rid +
			    ", result = " + result + 
			    (ok ? " (ok)" : " (rejected)"));
		i++;
	    }
	    if (i>0) userMessage(LOG_SUCCESS,"Server: finished: tags=" + i);
	    return FTAActor.KEEPALIVE;
	}
	catch(FTAException e) {
	    userMessage(LOG_ERROR,"FTAException occurred");
	    e.printStackTrace();
	    return FTAActor.TERMINATE;
	}
    }
    
    /**
     * mandatory init method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incoreect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	// nothing needed
    }
    
    /**
     * mandatory empty constructor
     */
    public Server() {
	super();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
