// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: Dyn.java,v 1.2 2001/06/21 14:57:13 veit Exp $

package jfta.demo.dynam;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.http.*;

/**
 * This actor just prints a message each time it is called
 * @author Holger Veit
 * @version $Id: Dyn.java,v 1.2 2001/06/21 14:57:13 veit Exp $
 */
public class Dyn extends FTAActor implements FTAActorI {
    
    private int _ninst;
    private int _gen;
    private int _phase;

    /**
     * mandatory eval method. This simply dumps out "Dyn World"
     * @return FTAActor.TERMINATE if the actor terminates after this
     *   routine has been called, FTAActor.KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {
	if (_ninst > 0) {
	    // will create as many ensembles with a new actor instance 
	    // of itself as the _ninst variable specifies.
	    // if _ninst is 0 don't reproduce self but just print a single
	    // aliveness message and idle.
	    try {
		switch (_phase) {
		case 0: // attempt to create an ensemble
		    for (int i = 0; i<_ninst; i++) {
			String ename = "en_"+getName()+"_"+i;
			addEnsemble(ename,0);
			userMessage(LOG_SUCCESS,"Add dynamic ensemble "+ename);
		    }
		    // if successful, go to phase 1
		    _phase++;
		    return KEEPALIVE;
		case 1: // create actors on the ensembles
		    for (int i=0; i<_ninst; i++) {
			String acname = getName()+"_"+i;
			String ename = "en_"+acname;
			Dyn ac = new Dyn(_ninst-1,_gen+1);
			addActor(ename,acname,ac);
			userMessage(LOG_SUCCESS,"Add dynamic actor "+acname);
		    }
		    // if successful, finalize
		    _ninst = 0;
		    _phase = 0;
		}		    
	    }
	    catch (FTAException e) {
		// ignore that, results from attempt to create an
		// actor on a non-existing ensemble
		userMessage(LOG_ERROR,"Error "+e+" actor="+getName());
	    }
	} else {
	    // report name and idle
	    if (_phase == 0) {
		userMessage(LOG_SUCCESS,
			    "Gen"+_gen+" Actor "+getName()+" is alive");
		_phase++;
	    }
	}
	return KEEPALIVE;
    }

    /**
     * mandatory init method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incoreect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	if (args.length < 2)
	    throw new FTAArgumentException("Expect 2 args: number of instances + generation");
	_ninst = _svc.atoi(args[0],2);
	_gen = _svc.atoi(args[1],3);
    }
    
    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHtmlFile f = new FTAHtmlFile("Actor "+getName());
	f.p("My ninst is "+_ninst);
	f.p("My gen is "+_gen);
	f.p("My phase is "+_phase);
	return f.finish();
    }


    /**
     * mandatory empty constructor
     */
    public Dyn() {
	super();
    }

    public Dyn(int ninst,int gen) {
      super();
      _ninst = ninst;
      _gen = gen;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * java-mode: t
 * font-lock-mode: t
 * End: 
 *
 */
