// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: MultiI.java,v 1.3 2001/06/21 14:57:16 veit Exp $

package jfta.demo.multi;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.base.*;

/**
 * This actor injects a given number of tags into the system, after
 * it has been stabilized (tagboard time of "tb" >= 10)
 * @author Holger Veit
 * @version $Id: MultiI.java,v 1.3 2001/06/21 14:57:16 veit Exp $
 */
public class MultiI extends FTAActor implements FTAActorI {
    
    private final static FTAService _svc = FTAService.Instance();
    private boolean[] _scratchpad = new boolean[32];

    /**
     * number of hosts
     */
    private int _nhosts;
    
    /**
     * number of tags to be injected
     */
    private int _tagcount;
    
    /**
     * timer to wait for TB to be stabilized
     */
    private long _lastSeen;

    /**
     * mandatory eval() method
     * @return TERMINATE if the actor terminates after this
     *   routine has been called, KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {

	// wait for time to pass
	long now = getTagboardTime("tb",RELATIVE);
	if (now <= _lastSeen)
	    return KEEPALIVE;
	_lastSeen = now;

	// check whether to inject data
	if (now < 10)
	    return KEEPALIVE;

	// inject data
	userMessage(LOG_SUCCESS,"Injecting "+_tagcount+" tags");
	for (int i=0; i<_tagcount; i++) {
	    putTag("tb",new Individual(_nhosts));
	}

	// job done
	userMessage(LOG_SUCCESS,"Injector Terminating");
	return TERMINATE;
    }
    
    /**
     * mandatory init() method
     * @param args list of arguments passed by <code>addActor</code>
     * to the newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code
     * if some argument is incorrect for this type of actor
     */
    public void init(String[] args) throws FTAArgumentException {

	if (args.length==0)
	    throw new FTAArgumentException("MultiI nhosts [tagcnt]");
	_nhosts = _svc.atoi(args[0],2);
	_tagcount = 100;

	if (args.length >= 2) {
	    _tagcount = _svc.atoi(args[1],100);
	}
    }
    
    /**
     * mandatory
     */
    public MultiI() {
	super();
    }

    /**
     * custom constructors
     */

    public MultiI(int nhosts) {
	this(nhosts,100);
    }

    public MultiI(int nhosts,int tagcount) {
	super();
	_nhosts = nhosts;
	_tagcount = tagcount;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
