// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: MultiInit.java,v 1.3 2001/06/21 14:57:16 veit Exp $

package jfta.demo.multi;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.base.FTAService;

/**
 * This init actor simply creates an ensemble on *this host*
 * and runs the Multi actor which simply prints out "hello"
 * each time it is called.
 * @author 
 * @version $Id: MultiInit.java,v 1.3 2001/06/21 14:57:16 veit Exp $
 */
public class MultiInit extends FTAInitActor implements FTAActorI {

    private int tagcount;

    /**
     * mandatory init routine
     * @return FTAActor.TERMINATE if the actor terminates after this
     *   routine has been called, FTAActor.KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {
	try {
	    userMessage(LOG_SUCCESS,"User: InitActor started");
	    
	    /*
	     * obtain list of hosts and distribute over all
	     * available ones
	     */
	    String[] hosts = getHosts();
	    int i;
	    int nhosts = hosts.length;

	    if (nhosts < 2) {
		userMessage(LOG_ERROR,
			    "User: this application needs >=2 hosts");
		return FTAActor.TERMINATE;
	    }

	    for (i=0; i<nhosts; i++) {
		addHost(hosts[i]);
		userMessage(LOG_SUCCESS,"User: Added host \"" + 
			    hosts[i]+"\"");
	    }

	    // add the work ensemble, main is on first host
	    addEnsemble("en",hosts[0],1);
	    userMessage(LOG_SUCCESS,
			"User: Added ensemble \"en\"");

	    // add the work tagboard, main is on first host
	    addTagboard("tb",hosts[1],1);
	    userMessage(LOG_SUCCESS,
			"User: Added ensemble \"tb\"");

	    // add actors to ensemble(s)
	    for (i=0; i<nhosts; i++) {
		addActor(hosts[i],"en","ga"+i,new Multi(i));
		userMessage(LOG_SUCCESS,
			    "User: Added actor \"ga"+i+"\" to \"en\"");
	    }

	    // visualizer
	    addActor(hosts[0],"en","visu",new MultiV());
	    userMessage(LOG_SUCCESS,
			"User: Added actor \"visu\" to \"en\"");

	    // injector, inject NN tags
	    addActor(hosts[1],"en","inj",new MultiI(nhosts,tagcount));
	    userMessage(LOG_SUCCESS,
			"User: Added actor \"inj\" to \"en\"");

	    // add sync
	    addETSync("en","tb");
	}
	catch(FTAException e) {
	    e.printStackTrace();
	    userMessage(LOG_ERROR,
			"Exception in MultiInit actor, exc="+e);
	    return FTAActor.TERMINATE;
	}
	userMessage(LOG_SUCCESS,"User: InitActor terminating");
	return FTAActor.TERMINATE;
    }
    
    /**
     * mandatory init method.
     * This requires *this* host name.
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incorrect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	FTAService svc = FTAService.Instance();
	if (args.length==1)
	    tagcount = _svc.atoi(args[0],100);
	else
	    tagcount = 100;
    }
    
    /**
     * mandatory empty constructor
     */
    public MultiInit() {
	super();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
