// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: OutputServer.java,v 1.2 2001/06/21 14:57:17 veit Exp $

package jfta.demo.output;

import jfta.base.*;
import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.http.*;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * This server openes a server socket on a public port 16164
 * and listens for incoming input which is simply sent to
 * System.out
 * @author Holger Veit
 * @version $Id: OutputServer.java,v 1.2 2001/06/21 14:57:17 veit Exp $
 */
public class OutputServer {
    
    /**
     * public socket number
     */
    public final static int PUBLICSOCK = 16164;

    /**
     * socket for communication
     */
    private ServerSocket _sock;
    
    /**
     * connection count
     */
    private int _conncnt;

    /**
     * The handleSession method does all the communication of a new
     * connection. In this case, it just receives
     * data and prints it to stdout.
     */
    private void handleSession(BufferedReader in,PrintStream out)
	throws Exception {

	int mycnt = ++_conncnt;

	// perform "open" protocol
	// incoming connection is supposed to send "open", and
	// this code replies with "ok". After that, both partners
	// are locked.
	String line = in.readLine().toLowerCase();
	if (!line.startsWith("open"))
	    return;
	out.println("ok "+mycnt);

	for (;;) {
	    line = in.readLine().toLowerCase();
	    if (line.startsWith("close "+mycnt)) {
		return;
	    } else if (line.startsWith("exit")) {
		System.exit(0);
	    }

	    // echo the output back to client
	    out.println("Echo: "+line);
	}
    }

    /**
     * the Handler thread manages a single session.
     * It listens for input data and if unblocked, passes it
     * to the method handleSession(). This method will
     * return after the communication is closed.
     */
    private class HandlerThread extends Thread {
	private Socket _conn;
	public HandlerThread(Socket conn) {
	    super();
	    _conn = conn;
	}
				    
	public void run() {
	    BufferedReader in;
	    PrintStream out;
	    try {
		in = new BufferedReader(new
		    InputStreamReader(_conn.getInputStream()));
		out = new PrintStream(_conn.getOutputStream());
		handleSession(in,out);
		_conn.close();
	    } catch (Exception e) {}
	}
    }

    /**
     * The server thread maintains the public server socket.
     * it blocks until a new connection is accepted and
     * then spawns off a new handler thread.
     */
    private class ServerThread extends Thread {
	private ServerSocket _sock;
	public ServerThread(ServerSocket s) {
	    super();
	    _sock = s;
	}
	
	/**
	 * the run method of the server thread continuously listens
	 * for new connections, and if connected, will spawn a new
	 * handler thread.
	 */
	public void run() {
	    try {
		while (_sock != null) {
		    Socket conn = _sock.accept();
		    HandlerThread thr2 = new HandlerThread(conn);
		    thr2.start();
		}
	    } catch (Exception e) {}
	}
    }

    /**
     * main program
     */
    public static void main(String[] args) {
	new OutputServer();
    }
    
    /**
     * mandatory empty constructor
     */
    public OutputServer() {
	try {
	    // create a server socket to listen on
	    _sock = new ServerSocket(PUBLICSOCK,10);
	}
	catch(IOException e) {
	    e.printStackTrace();
	    System.exit(1);
	}

	// create a server thread to process an incoming connection
	ServerThread thr = new ServerThread(_sock);
	thr.start();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 *
 */
