// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.ping;

import jfta.actors.*;
import jfta.exception.*;
import jfta.hash.*;
import jfta.tags.*;
import java.rmi.RemoteException;


public class PongActor extends FTAActor implements FTAActorI {
  private int count;
  private TagboardData td0,td1;
  
  public void init(String args[])throws FTAArgumentException{
    count = 0;
    td0 = new TagboardData("tbd0","read",this);
    td1 = new TagboardData(args[0],"write",this);
  }

    /**
       prints messages 
    */
    private void printActor(String msg){
	System.out.println(getName() + "["+count+"]: " + msg);
    }
    
    /**
       increase actor calls count
    */
    private void incTime(){
	++count;
    }

    /**
       sends a pong tag
    */
    private void sendPong(String[] timestr)
	throws FTATagException,FTAArgumentException,FTAUnknownObjectException{
	td1.putTag("jfta.demo.ping.PongTag",timestr);
    }
    
    /**
       receives ping tag and extracts information
     */
    private void recvPing()
	throws FTATagException,FTAArgumentException,FTAUnknownObjectException{
	PingTag tag = null;
	String[] timestr = null;
	FTATagIterator it = 
	    td0.getTagIterator(new FTATagPattern("jfta.demo.ping.PingTag"));
	try{
	    while(it.hasNext()){
				
		tag = (PingTag) it.next();
	
		timestr = extractTime(tag);

		sendPong(timestr);
	    }
	}
	catch(FTATagboardFlipException e){
	    printActor("tagboard has flipped.");
	} 
    }

    private String[] extractTime(PingTag tag){    
	if(tag == null)
	    printActor("PingTag is null");

	String content = tag.getContent();
	String[] timestr = new String[2];

	int idx = content.indexOf(":");
	if(idx > 0){
	    timestr[0] = content.substring(0,idx);
	    timestr[1] = content.substring(idx+1,content.length());
	}
	return timestr;
    } 
    
    public int eval() throws FTAEvalActorException{
	try { 
	    // increase time
	    incTime();
	    
	    // recv ping, send pong
	    if(td0.hasFlipped())
		{
		    recvPing(); 
		    //     flipOrder("tbd0");
		}
	    
	    // printActor("called.");
	    
	    if(count < 10000)
		return FTAActor.KEEPALIVE;
	    else {
		printActor("good bye.");
		return FTAActor.TERMINATE;
	    }
	}
	catch(Exception e) {
	    System.out.println("Exception raised: " + e);
	    e.printStackTrace();
	    return FTAActor.TERMINATE;
	}
    }
    public PongActor(){super();}
}
