// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: Display.java,v 1.2 2001/06/21 14:57:18 veit Exp $

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.base.*;
import jfta.base.gui.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;


/**
  * This class visualizes a List of MapObjTag objects. It implements
  * the DisplayI interface
  * @author Holger Veit
  * @version $Id: Display.java,v 1.2 2001/06/21 14:57:18 veit Exp $
  */
public class Display extends Object 
    implements DisplayI,ActionListener,AdjustmentListener,ItemListener,Serializable {

    /**
     */
    private FTAActor actor;
    private Frame top;
    private MapCanvas canvas;
    private Scrollbar vbar,hbar;
    private int xPos,yPos;
    private transient FTAGUIPrintManager prt;
    private double scale;
    private int gridscale,gridtick;

    /**
     * mandatory empty constructor
     */
    public Display() {
	actor = null;
	xPos = 0;
	yPos = 0;

	FTAService _svc = FTAService.Instance();
	gridscale = _svc.getIntegerProperty("grid_scale_dist",100);
	gridtick = _svc.getIntegerProperty("grid_tick_dist",10);
    }

    /**
     * Interface DisplayI:
     */
    public void setProperty(String name,double param) {
	canvas.setProperty(name,param);
    }

    /**
     * Interface DisplayI:
     */
    public double getProperty(String name) {
	return canvas.getProperty(name);
    }

    /**
     * Interface DisplayI:
     * add dynamic object
     */
    public void addObject(DynObjectI dynobj) {
	canvas.addObject(dynobj);
    }

    /**
     * Interface DisplayI:
     * remove dynamic object
     */
    public void removeObject(DynObjectI dynobj) {
	canvas.removeObject(dynobj);
    }

    /**
     * Interface DisplayI:
     * remove all dynamic objects
     */
    public void removeAllObjects() {
	canvas.removeAllObjects();
    }

    /**
     * Interface DisplayI:
     * add dynamic object
     */
    public void modifyObject(ModObjectI mobj) {
	canvas.modifyObject(mobj);
    }

    /**
     * Interface DisplayI:
     * This method is called when the data set changed
     */
    public void update() {
	canvas.update();
    }
    
    /**
     * Interface DisplayI:
     * This method is called by the DisplayActor to check whether the
     * Displayer windows has been closed (true, if so)
     */
    public boolean hasTerminated() {
	return canvas.hasTerminated();
    }
    
    /**
     * Interface DisplayI:
     * This is the method called to initialize the Display
     * display completely
     */
    public void init(FTAActor ac,String windowtitle,int xs,int ys) {
	actor = ac;
	top = new Frame(windowtitle);
	prt = new FTAGUIPrintManager(top);
	top.setLayout(new BorderLayout());

	add_menus();
      
	canvas = new MapCanvas(xs,ys);
	hbar = new Scrollbar(Scrollbar.HORIZONTAL);
	hbar.setMaximum(xs);
	hbar.addAdjustmentListener(this);
	vbar = new Scrollbar(Scrollbar.VERTICAL);
	vbar.setMaximum(ys);
	vbar.addAdjustmentListener(this);
	top.add(canvas,"Center");
	top.add(hbar,"South");
	top.add(vbar,"East");

	top.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    exit();
		}
	    });
      
	top.pack();
	top.setLocation(100,100);

	top.setVisible(true);
    }

    /**
     * create the menu bar
     */
    private void add_menus() {
	MenuBar mb = new MenuBar();

	Menu fmenu = new Menu("File");
	FTAGUIMenuItem it = 
	    new FTAGUIMenuItem(fmenu,"Print Setup",KeyEvent.VK_R,this);
	it.setEnabled(false);
	it = new FTAGUIMenuItem(fmenu,"Print",KeyEvent.VK_P,this);
	it.setEnabled(false);
	fmenu.addSeparator();
	new FTAGUIMenuItem(fmenu,"Exit",KeyEvent.VK_X,this);
	mb.add(fmenu);

	Menu vmenu = new Menu("View");
	new FTAGUIMenuItem(vmenu,"Zoom In",KeyEvent.VK_I,this);
	new FTAGUIMenuItem(vmenu,"Zoom Out",KeyEvent.VK_O,this);
	vmenu.addSeparator();
	new FTAGUIMenuItem(vmenu,"Grid Settings",KeyEvent.VK_S,this);
	new FTAGUICheckboxMenuItem(vmenu,"Show Names",false,KeyEvent.VK_N,this);
	mb.add(vmenu);

	top.setMenuBar(mb);

	Menu hmenu = new Menu("Help");
	new FTAGUIMenuItem(hmenu,"Command Help",KeyEvent.VK_H,this);
	new FTAGUIMenuItem(hmenu,"Mapfile Documentation",KeyEvent.VK_M,this);
	mb.setHelpMenu(hmenu);
    }
    
    /**
     * Interface ActionListener:
     * called by the Button/Menu Handler
     */
    public void actionPerformed(ActionEvent e) {
	Object obj = e.getSource();
	if (obj instanceof FTAGUIMenuItem) {
	    FTAGUIMenuItem m = (FTAGUIMenuItem)obj;
	    if (m.isequal("Zoom In")) {
		zoomin();
	    } else if (m.isequal("Zoom Out")) {
		zoomout();
	    } else if (m.isequal("Grid Settings")) {
		gridsettings();
	    } else if (m.isequal("Print Setup")) {
		printsetup();
	    } else if (m.isequal("Print")) {
		print();
	    } else if (m.isequal("Exit")) {
		exit();
	    } else if (m.isequal("Command Help")) {
		help("/jfta/demo/simrob/dishelp.txt");
	    } else if (m.isequal("Mapfile Documentation")) {
		help("/jfta/demo/simrob/maphelp.txt");
	    }
	}
    }

    /**
     * Interface AdjustmentListener:
     * called by Scrollbar Handler
     */
    public void adjustmentValueChanged(AdjustmentEvent e) {
	Adjustable adj = e.getAdjustable();
	if (adj.getOrientation()==Adjustable.HORIZONTAL)
	    xPos = e.getValue();
	else
	    yPos = e.getValue();
	// redraw canvas
	canvas.redraw(xPos,yPos);
    }

    /**
     * Interface ItemListener
     */
    public void itemStateChanged(ItemEvent e) {
	Object src = e.getSource();
	if (src instanceof FTAGUICheckboxMenuItem) {
	    FTAGUICheckboxMenuItem cb = (FTAGUICheckboxMenuItem)src;
	    canvas.setProperty("names",cb.getState() ? 1.0 : -1.0);
	}
    }

    /**
     * handle zoomin option
     */
    private void zoomin() {
	double scale = canvas.getProperty("scale") / 0.9;
	canvas.setProperty("scale",scale);
    }

    /**
     * handle zoomout option
     */
    private void zoomout() {
	double scale = canvas.getProperty("scale") * 0.9;
	canvas.setProperty("scale",scale);
    }

    /**
     * handle grid option
     */
    /*package*/ void grid(boolean onoff,int sc,int ti) {
	gridscale = sc;
	gridtick = ti;
	if (onoff) {
	    canvas.setProperty("gridscale",(double)sc);
	    canvas.setProperty("gridtick",(double)ti);
	} else {
	    canvas.setProperty("gridscale",0.0);
	    canvas.setProperty("gridtick",0.0);
	}
    }
    
    /**
     * Grid Dialog
     */
    private class DisplayGridDialog extends Dialog implements ActionListener {
	private GridBagLayout lm;
	private GridBagConstraints gbc;
	private FTAGUITextField sc,ti;
	private FTAGUICheckbox en;
	private Display disp;
	private void addComponent(Component c,int gridx,int gridy,
				  int gridwidth,int gridheight) {
	    gbc.gridx      = gridx;
	    gbc.gridy      = gridy;
	    gbc.gridwidth  = gridwidth;
	    gbc.gridheight = gridheight;
	    gbc.fill       = GridBagConstraints.NONE;
	    gbc.anchor     = GridBagConstraints.CENTER;
	    lm.setConstraints(c,gbc);
	    add(c);
	}

	public DisplayGridDialog(Frame top,Display disp,int scale,int tick) {
	    /*   0       1
	     * 0 Grid Settings:
	     * 1 Scale:  [    ]
	     * 2 Ticks:  [    ]
	     * 3
	     * 4 [x] enable Grid
	     * 5
	     * 6 [Ok]    [Cancel]
	     */
	    super(top,"Grid Settings");
	    this.disp = disp;
	    setLayout(lm = new GridBagLayout());
	    gbc = new GridBagConstraints();
	    addComponent(new Label("Grid Settings:"),0,0,2,1);
	    addComponent(new Label("Scales:"),0,1,1,1);
	    addComponent(sc=new FTAGUITextField(Integer.toString(scale),6),
			 1,1,1,1);
	    addComponent(new Label("Ticks:"),0,2,1,1);
	    addComponent(ti=new FTAGUITextField(Integer.toString(tick),6),
			 1,2,1,1);
	    addComponent(en=new FTAGUICheckbox("Enable grid",false),0,4,2,1);
	    Panel p = new Panel();
	    p.setLayout(new FlowLayout());
	    p.add(new FTAGUIButton("Ok",this));
	    p.add(new FTAGUIButton("Cancel",this));
	    addComponent(p,0,6,2,1);
	    pack();
	}

	private int getnum(String str) {
	    try {
		return Integer.parseInt(str);
	    }
	    catch (NumberFormatException e) {
		return 0;
	    }
	}

	private boolean getOnoff() {
	    return en.getState();
	}

	public void actionPerformed(ActionEvent e) {
	    Object src = e.getSource();
	    if (src instanceof FTAGUIButton) {
		FTAGUIButton b = (FTAGUIButton)src;
		if (b.getLabel().equals("Ok")) {
		    disp.grid(getOnoff(),getnum(sc.getText()),getnum(ti.getText()));
		}
	    }
	    setVisible(false);
	}
    }

    /**
     * handle grid option
     */
    private void gridsettings() {
	DisplayGridDialog dgd = new DisplayGridDialog(top,this,
						      gridscale,gridtick);
	dgd.setVisible(true);
    }

    /**
     * handle print option
     */
    private void print() {
	prt.print(canvas);
    }

    /**
     * handle print setup option
     */
    private void printsetup() {
	prt.printSetup();
    }

    /**
     * handle help option
     */
    private void help(String helpfile) {
	FTAGUIHelpWindow hw = new FTAGUIHelpWindow(top,"Command Help",helpfile);
	hw.show();
    }

    /**
     * handle exit option
     */
    private void exit() {
	canvas.terminate();
	top.setVisible(false);
	top.dispose();
    }

    public static void main(String[] args) {
	Display m = new Display();
	m.init(null,"DisplayTest",640,480);
    }
}
