// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// -*- java -*-

package jfta.demo.simrob;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.print.*;
import java.util.*;

import jfta.base.*;
import jfta.actors.*;

/**
 * This is a canvas widget to display the map
 */
public class MapCanvas extends Panel implements DisplayI,Printable
{
  // double buffering support
  private Image offscreen;
  private Dimension dim;
  private Hashtable objs;
  private int xPos, yPos;
  private int xuser,yuser;
  private boolean term;
  private double scale;
  private static int gridscale,gridtick;
  private static BasicStroke gridscalewidth,gridtickwidth;
  private static Color gridscalecolor=null,gridtickcolor=null;
  private static Color canvascolor;
  private int flags;

  public MapCanvas(int xs,int ys) {
    super();

    setLayout(null);

    xuser = xs;
    yuser = ys;
    scale = 1.0;
    dim = new Dimension(xs,ys);
    term = false;
    xPos = 0;
    yPos = 0;
    objs = new Hashtable();
    gridscale = gridtick = 0;

    FTAService _svc = FTAService.Instance();
    canvascolor = _svc.getColorProperty("map_canvas_bgcolor",Color.white);
    setBackground(canvascolor);

    offscreen = null;

    if (gridscalecolor==null) {
      gridscalecolor = _svc.getColorProperty("grid_scale_color",Color.lightGray);
      gridscalewidth = new BasicStroke(_svc.getFloatProperty("grid_scale_width",
							     (float)2.0));
      gridtickcolor = _svc.getColorProperty("grid_scale_color",Color.lightGray);
      gridtickwidth = new BasicStroke(_svc.getFloatProperty("grid_tick_width",
							    (float)1.0));
    }
  }
  
  /**
    * Interface DisplayI:
    * add dynamic object
    */
  public void addObject(DynObjectI dynobj) {
    String name = dynobj.getName();
    if (dynobj instanceof MapSwallObjTag) {
      MapSwallObjTag swall = (MapSwallObjTag)dynobj;

      xuser = swall.getX();
      yuser = swall.getY();
      //System.out.println("user="+xuser+" "+yuser);
      //System.out.println("dim="+dim);

      double xscale = (double)(dim.width-10) / xuser;
      double yscale = (double)(dim.height-10) / yuser;
      //System.out.println("Scale=("+xscale+","+yscale+")");
      scale = Math.min(xscale,yscale);
    }

    if (objs.get(name) != null) {
      objs.remove(name);
    }
    objs.put(name,dynobj);
    
    //System.out.println("Display: dynobj added:");
  }
  
  /**
    * Interface DisplayI:
    * remove dynamic object
    */
  public void removeObject(DynObjectI dynobj) {
    objs.remove(dynobj.getName());
    
    //System.out.println("Display: dynobj removed");
  }
  
  /**
    * Interface DisplayI:
    * remove all dynamic objects
    */
  public void removeAllObjects() {
    objs.clear();
    
    //System.out.println("Display: all dynobjs removed");
  }

  /**
    * Interface DisplayI:
    * modify an object
    */
  public void modifyObject(ModObjectI mobj) {
    DynObjectI obj = (DynObjectI)objs.get(mobj.getName());
    if (obj != null)
      obj.modify(mobj);
  }
  
  /**
    * Interface DisplayI:
    * This method is called when the data set changed
    */
  public void update() {
    //System.out.println("update()");
    Graphics g = getGraphics();
    paint(g);
    //repaint();
  }
    
  /**
    * Interface DisplayI:
    * not used
    */
  public void init(FTAActor ac,String title,int xs,int ys) {}

  /**
    * Interface DisplayI:
    * set a property
    */
  public void setProperty(String name,double param) {
    if (name.equals("gridscale")) {
      gridscale = (int)param;
    } else if (name.equals("gridtick")) {
      gridtick = (int)param;
    } else if (name.equals("scale")) {
      scale = param;
    } else if (name.equals("names")) {
      if ( param > 0.0)
	flags |= MapObjTag.DRAWNAME;
      else
	flags &= ~MapObjTag.DRAWNAME;
    }
    update();
  }

  /**
    * Interface DisplayI:
    * get a property
    */
  public double getProperty(String name) {
    if (name.equals("gridscale")) {
      return (double)gridscale;
    } else if (name.equals("gridtick")) {
      return (double)gridtick;
    } else if (name.equals("scale")) {
      return (double)scale;
    } else if (name.equals("name")) {
      return (flags & MapObjTag.DRAWNAME) != 0 ? 1.0 : -1.0;
    } else return 0.0;
  }

  /**
    * Interface DisplayI:
    */
  public boolean hasTerminated() {
    return term;
  }

  /**
    * Interface DisplayI:
    */
  public void terminate() {
    term = true;
  }

  public void redraw(int xnew,int ynew) {
    xPos = xnew;
    yPos = ynew;
    update();
  }

  private void drawGrid(Graphics2D g) {
    if (gridscale == 0)
      return;

    int xi,yi;
    int xBeg = 0;
    int yBeg = 0;
      
    // draw scales
    g.setColor(gridscalecolor);
    g.setStroke(gridscalewidth);
    for (xi = 0; xi <xuser; xi += gridscale)
      g.drawLine(xi,0,xi,yuser);
    for (yi = 0; yi <yuser; yi += gridscale)
      g.drawLine(0,yi,xuser,yi);

    // draw ticks
    if (gridtick != 0) {
      g.setColor(gridtickcolor);
      g.setStroke(gridtickwidth);
      for (xi = 0; xi <xuser; xi += gridtick)
	g.drawLine(xi,0,xi,yuser);
      for (yi = 0; yi <yuser; yi += gridtick)
	g.drawLine(0,yi,xuser,yi);
    }
  }

  public void paint(Graphics g) {
    //System.out.println("paint(g)");
    Dimension s = getSize();

    if (offscreen == null) {
      offscreen = createImage(s.width,s.height);
    }
    Graphics2D og = (Graphics2D)offscreen.getGraphics();

    //og.setClip(0,0,s.width,s.height);
    og.setBackground(Color.white);
    og.clearRect(0,0,s.width,s.height);

    g.setColor(Color.white);
    g.clearRect(0,0,s.width,s.height);

    og.translate(5,5);
    og.setFont(new Font("sanserif",Font.PLAIN,(int)(12/scale)));
    og.scale(scale,scale);

    drawGrid(og);

    /* draw map objects */
    if (objs.size() == 0) {
      og.scale(1.0,1.0);
      og.setColor(Color.red);
      og.setFont(new Font("Arial",Font.PLAIN,18));
      og.drawString("No map available",30,30);
    } else {
      Enumeration it = objs.elements();
      while (it.hasMoreElements()) {
	DynObjectI obj = (DynObjectI)it.nextElement();
	obj.paint(og,flags);
      }
    }

    g.drawImage(offscreen,-xPos,-yPos,this);
    og.dispose();
  }

  public Dimension getPreferredSize() {
    return getMinimumSize();
  }

  public Dimension getMinimumSize() {
    return dim;
  }

  public Dimension getMaximumSize() {
    return dim;
  }

  /**
    * Printable Interface
    */
  public int print(Graphics g, PageFormat format,int index)
    throws PrinterException {
      if (index > 0)
	return Printable.NO_SUCH_PAGE;

      Graphics2D g2 = (Graphics2D)g;
      g2.translate(format.getImageableX(),format.getImageableY());
      paint(g2);

      return Printable.PAGE_EXISTS;
  }
}
