// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.base.*;
import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

import java.io.*;
import java.util.*;
import java.awt.*;

/**
  * This actor maintains the list of robots, and manages its loading
  * and unloading
  * @author Holger Veit, Andre Schmitz, Frank Schoenherr
  * @version $Id: RobotLoader.java,v 1.2 2001/06/21 14:57:19 veit Exp $
  */
public class RobotLoader extends FTAActor implements FTAActorI,Constants {

  private final static String WhoAmI = "RobotLoader";
  private final static FTAService _svc = FTAService.Instance();
  
  private Hashtable vehcmds = new Hashtable();

  // temporary
  private String robotname;
  private String filename;
  private String hostname;
  private String cmdTB;
  private String sensorTB;
  private String speedTB;
  private String configTB;
  private String robotEN;
  private String motoractor;
  
  private FTACallback cb;

  /**
    * return the name of a robot specific board
    */
  public static String getNamedObject(String name,int boardid) {
    switch (boardid) {
    case SENSORTB:
      return "SensorTB_" + name;
    case SPEEDTB:
      return "SpeedTB_" + name;
    case CMDTB:
      return "CommandTB_" + name;
    case ROBOTEN:
      return "RobotEN_" + name;
    case CONFTB:
      return "ConfigTB_" + name;
    default:
      return null;
    }
  }
  
  /**
    * assign a hostname for a new robot instance
    */
  private String assignRobotHost() {
    return _svc.getFQDNname();
  }
  
  /**
    * create structures for the new robot instance
    */
  private void createRobotStructures()
    throws FTACreationException,FTAUnknownObjectException {
      
      cmdTB = getNamedObject(robotname,CMDTB);
      speedTB = getNamedObject(robotname,SPEEDTB);
      sensorTB = getNamedObject(robotname,SENSORTB);
      configTB = getNamedObject(robotname,CONFTB);
      robotEN = getNamedObject(robotname,ROBOTEN);
      
      System.out.println("Try to add tagboards");
      addTagboard(cmdTB, hostname, 1);
      addTagboard(sensorTB, hostname, 1);
      addTagboard(speedTB, hostname, 1);
      addTagboard(configTB, hostname, 1);
      System.out.println("Tagboards added");
      
      System.out.println("Try to add ensemble");
      addEnsemble(robotEN, hostname, 1);
      System.out.println("Ensemble added");
  }
  
  /**
    * create the required service actors for robot
    */ 
  private void createRobotActors() 
    throws FTAUnknownObjectException, FTACreationException, 
    FTAArgumentException {

      // create a unique SimMotor-Actor name
      //addActor(hostname,robotEN,robotname,motoractor,null);
      addActor(robotname,motoractor,(String[])null);
  }
  
  /**
    * create a new instance of a robot
    */
  private void addRobot() throws FTAEvalActorException {

    try {
      // get a host for the new robot
      hostname = assignRobotHost();
      
      // create the associated tagboards and ensembles
      createRobotStructures();
      
      // configure the robot from file
      configureRobot();
      
      // create service actor for motor
      createRobotActors();
    }
    catch (FileNotFoundException e) {
      throw new FTAEvalActorException(WhoAmI + ": " + e.getMessage());
    }
  }
  
  /**
    * callback handler for messages
    */
  private class MsgCallback extends FTATBCallback {
    public MsgCallback(FTAActor ac,String tb) {
      super(ac,tb);
    }

    public int eval2() throws FTAEvalActorException {
      FTATagIterator iter = getTagIterator(msgTB,msgpattern);
      //String filename; 
      while (iter.hasNext()) {
	Object tag = iter.next();

	if (tag instanceof MsgAddVehTag) {
	  // set instance variables
	  MsgAddVehTag vt = (MsgAddVehTag)tag;
	  filename = vt.getFile();
	  robotname = vt.getVeh();
          userMessage("RobotLoader",filename + " : " + robotname); 
	  // add a new robot and its data structures
	  addRobot();

	} else if (tag instanceof MsgTerminateTag) {
	  userMessage(WhoAmI,"EXITING...");    
	  return FTAActor.TERMINATE;
	}
      }
      return FTAActor.KEEPALIVE;
    }
  }

  /**
    * mandatory eval method
    * @return TERMINATE if the actor terminates after this
    *   routine has been called, KEEPALIVE to have the actor
    *   be rescheduled in the next tick cycle.
    * @exception FTAEvalActorException thrown in case a fatal error occurred
    */
  public int eval() throws FTAEvalActorException {
    return cb.eval();
  }
  
  /**
    * mandatory init method
    * @param args list of arguments passed by <code>addActor</code> to the
    *   newly created actor. 
    * @exception FTAArgumentException to be thrown by the user code if some
    *   argument is incorrect for this type of actor
    * @return returns true, if the initialization was okay, false if there
    *   has been a problem not requiring an <code>FTAArgumentException</code>
    */
  public void init(String[] args) throws FTAArgumentException {
    userMessage("RobotLoader"," START"); 
  }
  
  /**
    * basic constructor
    */
  public RobotLoader() {
    super();

    cb = new MsgCallback(this,msgTB);

    // fill the command table
    vehcmds.put("vehicle",new VehicleCmd());
    vehcmds.put("initpos",new PosCmd());
    vehcmds.put("nosensors",new NoSensorsCmd());
    vehcmds.put("nomotors",new NoMotorsCmd());
    vehcmds.put("nolasers",new NoLasersCmd());
    vehcmds.put("shape",new ShapeCmd());     
    vehcmds.put("motoractor",new MotorAcCmd());     
    vehcmds.put("sensor",new SensorCmd());  
    vehcmds.put("laserscan",new LaserScanCmd());  
    vehcmds.put("motor",new MotorCmd());  
    vehcmds.put("size",new SizeCmd());  
  }

  /*******************************************************************/
  /* configuration section                                           */
  /*******************************************************************/

  // config tag data
  VConfVehiclelocTag configtag;

  // Vehicle Motor and US-Sensor-numbers
  private int no_sensors, no_motors, no_lasers;
  private int sensorcnt, motorcnt, lasercnt;

  /**
    * configure robot from the data in the config file
    */
  private void configureRobot() 
    throws LineParserException, FileNotFoundException,
    FTAUnknownObjectException,FTAArgumentException,FTATagException {

      configtag = new VConfVehiclelocTag();
      configtag.setVtype("V_Tank");

      sensorcnt = -1;
      lasercnt = -1;
      motorcnt = -1;
      userMessage("RobotLoader", filename + " : " +  robotname + " : " + vehcmds); 
      // read the specified file
      LineParser dummyvar = new LineParser(filename,robotname,vehcmds);

      // send configuration tag and finally flip the data
      System.out.println("Try to put config tag");
      putTag(configTB, configtag);
      flipOrder(configTB);

      userMessage("SimServer","Added new vehicle " + robotname);
  }

  /**
    * LineParsers for the ADD-Command VEHICLE
    */  
  private class VehicleCmd extends Object implements LineParse,Serializable {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	// ignore argument
        return true;
    }        
  }
  
  /**
    * LineParsers for the ADD-Command INITPOS
    */
  private class PosCmd extends Object implements LineParse,Serializable {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	int x = LineParser.getInt(st);
	int y = LineParser.getInt(st);
	double or = LineParser.getDouble(st);
	configtag.setPos(x,y,or);
        return true;
    }        
  }

  /**
    * LineParsers for the ADD-Command SIZE
    */
  private class SizeCmd extends Object implements LineParse,Serializable {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	int w = LineParser.getInt(st);
	int l = LineParser.getInt(st);
	configtag.setSize(w,l);
        return true;
    }        
  }

  /**
    * LineParsers for the ADD-Command NOLASERS
    */  
  private class NoLasersCmd extends Object implements LineParse,Serializable {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	
	no_lasers = LineParser.getInt(st);
	configtag.setLaserScanners(no_lasers);
	return true;
    } 
  }
  
  /**
    * LineParsers for the ADD-Command NOSENSORS
    */  
  private class NoSensorsCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	no_sensors = LineParser.getInt(st);
        configtag.setUsSensors(no_sensors);
        return true;
    } 
  }
  
  /**
    * LineParsers for the ADD-Command NOMOTORS
    */  
  private class NoMotorsCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	no_motors = LineParser.getInt(st);
        configtag.setMotors(no_motors);
        return true;
    }
  }   
  
  /**
    * LineParsers for the ADD-Command SHAPE
    */  
  private class ShapeCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
        configtag.setShape(LineParser.getString(st,true));
        return true;
    }
  }   
  
  /**
    * LineParsers for the ADD-Command MOTORACTOR
    */  
  private class MotorAcCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
        motoractor = LineParser.getString(st,true);
        return true;
    }
  }   

  /**
    * LineParsers for the ADD-Command SENSOR
    */  
  private class SensorCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {

        ++sensorcnt;
        if (sensorcnt >= no_sensors) {
	  LineParser.exception(st,"Too many US sensors");
	  return false;
	}
	
        try {
	  LineParser.getString(st,true); // Type US after keyword not needed

	  int num = LineParser.getInt(st);
	  int x = LineParser.getInt(st);
	  int y = LineParser.getInt(st);
	  int dir = LineParser.getInt(st);
	  int maxr = LineParser.getInt(st);
	  int minr = LineParser.getInt(st);
	  double err = LineParser.getDouble(st);
	  
	  // put the US-Sensor-Tag on the ConfigTB
	  putTag(configTB, new VConfUSTag(num,"KST_us",x,y,dir,maxr,minr,err));
	  // will flip when final VConfVehicleTag is sent
	}
	catch (FTAEvalActorException e) {
	  LineParser.exception(st,e.getMessage());
	  return false;
	} 
        return true;
    }
  }   
  
  /**
    * LineParsers for the ADD-Command LASERSCAN
    */  
  private class LaserScanCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {

	++lasercnt;
	if (lasercnt >= no_lasers) {
	  LineParser.exception(st,"Too many Laser scanners");
	  return false;
	}

        // Add a new Laserscanner  
        int num = LineParser.getInt(st);
        int x = LineParser.getInt(st);
        int y = LineParser.getInt(st);
        int dir = LineParser.getInt(st);
        int maxr = LineParser.getInt(st);
        int minr = LineParser.getInt(st);
        int lang = LineParser.getInt(st);
        int rang = LineParser.getInt(st);
        double step = LineParser.getDouble(st);
        double err = LineParser.getDouble(st);

	try {
	  // put the LaserScanner-Tag on the ConfigTB
	  putTag(configTB, new VConfLscanTag(num,"KST_lscan",
					     x,y,dir,
					     maxr,minr,lang,rang,
					     step,err));
	  // flip will be done when last VConfVeh Tag is submitted
	}
	catch(FTAException e) {
	  LineParser.exception(st,"Failed to call putTag");
	  return false;
	} 
        return true;
    }
  }

  /**
    * LineParsers for the ADD-Command MOTOR
    */  
  private class MotorCmd extends Object implements LineParse,Serializable {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {

	++motorcnt;
        if(motorcnt >= no_motors) { // Too many MOTORS defined 
	  LineParser.exception(st,"Too many motors");
	  return false;
	}
        System.out.println("before");
	LineParser.getString(st,true); // type after keyword not needed
	LineParser.getString(st,true); // style after type not needed
 
	int num = LineParser.getInt(st);
	int x = LineParser.getInt(st);
	int y = LineParser.getInt(st);
	int dir = LineParser.getInt(st);
	int diam = (int)LineParser.getDouble(st);
	int tpt = LineParser.getInt(st);
	int mtps = LineParser.getInt(st);
	int maxs = LineParser.getInt(st);
	int mins = LineParser.getInt(st);
	double err = LineParser.getDouble(st);
        System.out.println("after");
        try {
          //put the Motor-Tag on the configTB
          putTag(configTB,new VConfMotLayoutTag(num,x,y,dir,
						diam,tpt,mtps,
						maxs,mins,err));
	  // flip is done when last VConfVehCmd is put
        }
	catch(FTAException e) {
	  LineParser.exception(st,"Failed to call putTag");
	  return false;
	} 
        return true;
    }
  }  
}
