// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// this is a remote communication object for the SimServer.
// $Id: SimServerRemote.java,v 1.2 2001/06/21 14:57:20 veit Exp $

package jfta.demo.simrob;

import java.rmi.registry.*;
import java.rmi.Remote;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.util.LinkedList;

import jfta.exception.*;
import jfta.base.*;

/**
  * This class manages the outside communication with the SimServer Actor
  * @author Holger Veit,
  * @version $Id: SimServerRemote.java,v 1.2 2001/06/21 14:57:20 veit Exp $
  */
public class SimServerRemote extends UnicastRemoteObject implements SimServerI {
    private final static FTAService _svc = FTAService.Instance();
    private String _myurl = null;
    private LinkedList _cmdlist = null;
  
    /**
     * Submit a command string to the simServer
     * @param cmd command line
     * @exception RemoteException thrown in case of communation failure
     */
    public void sendCmd(String cmd) throws RemoteException {
	_cmdlist.addLast(cmd);
    }
  
    /**
     * get URL this server is listening on (diagnostic, lifeness test)
     * @return URL
     *  @exception RemoteException thrown in case of communation failure
     */
    public String getURL() throws RemoteException {
	return _myurl;
    }

    /**
     * retrieve (destructively) the first available command from the list
     * @return String or null
     */
    public synchronized String getCmd() {
	return available() ? (String)_cmdlist.removeFirst() : null;
    }
  
    /**
     * check whether at least one item is available
     */
    public synchronized boolean available() {
	return _cmdlist.size() > 0;
    }

    /** 
     * construct an URL for this object
     * @param hostname FQDN name of this host
     * @param instname optional instance name for multiple channels
     * @return the constructed URL
     */
    public static String makeURL(String hostname,String instname) {
	return "rmi://" + 
	    hostname + 
	    "/fta/simrob/" +
	    (instname!=null ? instname : "inst0");
    }

    /**
     * retrieve a remote instance of the SimServerRemote object for access
     * @param hostname FQDN name of this host
     * @param instname optional instance name for multiple channels
     * @return a usable SimServerI object
     */ 
    public static SimServerI getRemote(String hostname,String instname) 
	throws FTAUnknownObjectException {
	FTAService svc = FTAService.Instance();
	return (SimServerI)svc.getRemoteFromCache(makeURL(hostname,instname));
    }
    
    /**
     * register the local server object at the local registry.
     */
    private void register() throws FTARegistryException {
	try {
	    Naming.rebind(_myurl,(Remote)this);
	}
	catch (Exception e) {
	    e.printStackTrace();
	    throw new FTARegistryException("no registry at " +
					   _svc.getFQDNname() + 
					   " url=" + _myurl);
	}
    }
    
    /**
     * unregister the local server from the local registry
     */
    public void unregister() throws FTARegistryException {
	try {
	    Naming.unbind(_myurl);
	}
	catch (Exception e) {
	    throw new FTARegistryException("no registry at " +
					   _svc.getFQDNname());
	}
    }
    
    /**
     * constructor: specify an optional instance name
     */
    public SimServerRemote(String instname) throws RemoteException {
	super();
	_myurl = makeURL(_svc.getFQDNname(),instname);
    
	_cmdlist = new LinkedList();
    
	try {
	    register();
	}
	catch(FTARegistryException e) {
	    e.printStackTrace();
	    System.exit(1);
	}
    }
}
