// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;

import java.awt.geom.*;

/**
  * This tag is used for describing the Motor(s) of the vehicle 
  * which is numbered VehNumber on the VConfTB
  * @author Frank Schoenherr
  * @version $Id: VConfMotLayoutTag.java,v 1.2 2001/06/21 14:57:20 veit Exp $
  */
public class VConfMotLayoutTag extends FTATag {

  private int    MotNumber;
  private int    TicksPTurn;
  private int    MaxTicksPSec;
  private double TickError;
  private int    posX,posY;
  private int    Direction;
  private int    WheelDiameter;
  private int    MaxSpeed;
  private int    MinSpeed;
  
  /** 
    * @return the number of this Motor
    */
  public int getNumber() {
    return MotNumber;
  }
  
  /**
    * @return the number of Ticks per wheel turn
    * 
    */ 
   public int getTicksPTurn() { 
      return TicksPTurn;
    }

  /**
    * @return the maximum of Ticks per second
    */
   public int getMaxTicksPSec() {
      return MaxTicksPSec;
   }

  /**
    * @return the error in the ticks of this motor
    */
  public double getTickError() {
    return TickError;
  }

  /**
    * @return the relativ position of this motor
    */
  public int getX() {
    return posX;
  }

  /**
    * @return the relativ position of this motor
    */
  public int getY() {
    return posX;
  }

  /**
    * @return the relativ drive direction of this motor
    */
  public int getDirection() {
    return Direction;
  }

  /**
    * @return the wheel diameter in mm of this motor
    */
  public int getWheelDiam()  {
    return WheelDiameter;
  }
 
  /**
    * @return the maximal speed in mm of this motor
    */
  public int getMaxSpeed()  {
    return MaxSpeed;
  } 

  /**
    * @return the minimal (reverse) speed in mm of this motor
    */
  public int getMinSpeed() {
    return MinSpeed;    
  } 
  
  /**
    * constructor
    */
  public VConfMotLayoutTag() {
    super(); 
  }

  /**
    * custom constructor
    */
  public VConfMotLayoutTag(int num,int x,int y,int dir,
			   int diam,int tpt,int mtk,
			   int maxsp,int minsp,double err) {
    super();

    MotNumber = num;
    posX = x;
    posY = y;
    Direction = dir;
    WheelDiameter = diam;
    TicksPTurn = tpt;
    MaxTicksPSec = mtk;
    MaxSpeed = maxsp;
    MinSpeed = minsp;
    TickError = err;
  }
}
