// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;

import java.awt.geom.*;

/**
  * This tag is used for describing the US-Sensor(s) of the vehicle 
  * which is numbered VehNumber on the VConfTB
  * @author Frank Schoenherr
  * @version $Id: VConfUSTag.java,v 1.2 2001/06/21 14:57:20 veit Exp $
  */
public class VConfUSTag extends FTATag {

 private int    VehNumber;
 private int    SensNumber;
 private String Type;
 private int    posX,posY;
 private int    Direction;
 private int    MaxRange;
 private int    MinRange;
 private double HasError;

  /** 
    * @return the number of this US-Sensor
    */
  public int getSensNumber() {
    return SensNumber;
  }

  /**
    * @return the Type of the Sensor: "KST_us" = is a US-Sensor
    * Don't name it getType(): this is already defined in FTATag!!!
    * The type is String and not enum like in the C-Version!!!
    */ 
   public String getSensType() { 
      return Type;
    }

  /**
    * @return the relativ position of this US-Sensor
    */
  public int getX() {
    return posX;
  }
  public int getY() {
    return posY;
  }
  
  /**
    * @return the relativ orientation (Scandirection) of this US-Sensor
    */
  public int getDirection() {
    return Direction;
  }

  /**
    * @return the maximal scanrange in mm of this US-Sensor
    */
  public int getMaxRange() {
    return MaxRange;
  }

  /**
    * @return the minimal scanrange in mm of this US-Sensor
    */
  public int getMinRange() {
    return MinRange;
  }

 
  /**
    * @return the error-value of this US-Sensor
    * the width of the Gaussian distribution is influenced by this value
    * and the reflection angle too (ask A. Schmitz)
    */
  public double getError() {
    return HasError;    
  } 

  /**
    * constructor
    */
  public VConfUSTag() {
    super(); 
  }

  /**
    * custom constructor
    */
  public VConfUSTag(int num,String typ,int x,int y,
		    int dir,int maxr,int minr, double err) {
    super();
    SensNumber = num;
    Type = typ;
    posX = x;
    posY = y;
    Direction = dir;
    MaxRange = maxr;
    MinRange = minr;
    HasError = err;
  }
}
