// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;

import java.awt.geom.*;
import java.math.*;

/**
  * This tag is used for describing the vehicle on the VConfTB
  * 
  * @author Frank Schoenherr
  * @version $Id: VConfVehiclelocTag.java,v 1.2 2001/06/21 14:57:20 veit Exp $
  */
public class VConfVehiclelocTag extends FTATag {

  private int    posX,posY;
  private int    width;
  private int    length;
  private double orientation;
  private int    NOUsSensors;
  private int    NOLaserScanner;
  private int    NOMotors;
  private String vtype;
  private String shape;
  
  public int getX() {
    return posX;
  }

  public int getY() {
    return posY;
  }

  /**
    * @return the Orientation of the Vehicle 
    */
  public double getOrient() {
    return orientation;
  }

  /**
    * @return the Width of the Vehicle
    */
  public double getWidth() {
    return width;
  }

  /**
    * @return the Length of the Vehicle
    */
  public double getLength() {
    return length;
  }

  /**
    * @return the number of US-Sensors of the Vehicle
    */
  public int getNOUsSensors() {
    return NOUsSensors;
  }

  /**
    * @return the number of LaserScanners of the Vehicle
    */
  public int getNOLaserScanner() {
    return NOLaserScanner;
  }

  /**
    * @return the number of Motors of the Vehicle
    */
  public int getNOMotors()  {
    return NOMotors;
  }
 
  /**
    * @return the Type of the Vehicle (=String and not enum!!)
    */
  public String getVType()  {
    return vtype;
  } 

  /**
    * constructor
    */
  public VConfVehiclelocTag() {
    super();
  }

  /**
    * set methods
    */
  public void setPos(int x,int y,double or) {
    posX = x;
    posY = y;
    orientation = or;
  }
  public void setSize(int w,int l) {
    width = w;
    length = l;
  }
  public void setUsSensors(int n) {
    NOUsSensors = n;
  }
  public void setLaserScanners(int n) {
    NOLaserScanner = n;
  }
  public void setMotors(int n) {
    NOMotors = n;
  }
  public void setVtype(String s) {
    vtype = s;
  }
  public void setShape(String s) {
    shape = s;
  }
}
