// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;


// Anmerkung die Umrechnung Laserplolarkoordinante->kartesisch 
// und die Berechnung des entspr. Winkelhistogramms kann auf
// einen anderen Akteur verteilt werden (kommunikation via TB)
/** Only for testing!!!!
  * 
  *Berechnet ob der Wandabstand (bei Flurverfolgung) ok!
  *Hierzu wird das Mittel der X-Achsenwerte der Laserscans betrachtet
  *Jeweils die rechte und linke Haelfte wird getrennt gemittelt.
  *Liegt die Differenz mehr als der Wert von AKZEPTABEL auseinander
  *wird ein entsprechnender Drehwinkel generiert.
  *Sehr weite X-Werte>MAXRANGE werden nicht betrachtet. Ausserdem
  * wird nur jeder zweite Scanpunkt genommen um Stoerungen durch
  *Personen etc. zu reduzieren.
  */

public class WandAbstand extends SimObject implements FTAActorI,Constants {

  private FTACallback cb;

  /**
    * mandatory empty constructor
    */
  public WandAbstand() {
    super();
  }

  /**
    * mandatory init method
    */
  public void init(String[] args) throws FTAArgumentException {

    initbase("WandAbstand",args[0]);
    cb = new MsgCallback(this);

    userMessage(WhoAmI, "START");
  }

  public WandAbstand(String veh) {
    super();
  }

  private class MsgCallback extends FTATBCallback {
    public MsgCallback(FTAActor ac) {
      super(ac,msgTB);
    }
    public int eval2() throws FTAEvalActorException {
      FTATagIterator iter = getTagIterator(tb,msgpattern); 
      while (iter.hasNext()) {   
	Object tag = iter.next(); 

	boolean term = false;
	if (tag instanceof MsgRemVehicleTag) {
	  MsgRemVehicleTag vt = (MsgRemVehicleTag)tag;
	  term = vehname.equals(vt.getVeh());
	} else
	  term = (tag instanceof MsgTerminateTag);

	if (term) {
          userMessage("WandAbstand_"+vehname,"EXITING...");
	  return TERMINATE;
	}
      }
      return KEEPALIVE;
    }
  }

  /**
    * mandatory eval method
    * @return TERMINATE if the actor terminates after this
    *   routine has been called, KEEPALIVE to have the actor
    *   be rescheduled in the next tick cycle.
    * @exception FTAEvalActorException thrown in case a fatal error occurred
    */
  public int eval() throws FTAEvalActorException {       

    // check for new message
    if (cb.eval() == TERMINATE)
      return TERMINATE;

    // control distance...to be done
    putTag(cmdTB,new VehCmdSetMotSpeedTag(SimObject.RIGHTWHEEL,100));
    putTag(cmdTB,new VehCmdSetMotSpeedTag(SimObject.LEFTWHEEL,100));
    System.out.println("Put tag");  
    flipOrder(cmdTB);

    return KEEPALIVE;
  }
}
