// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.test;

import jfta.actors.*;
import jfta.exception.*;
import jfta.hash.*;
import jfta.system.*;
import jfta.tags.*;
import java.rmi.RemoteException;


public class TestActor extends FTAActor implements FTAActorI {
    private String[] msgs ={"myname","hallo","welt"};
    private String[][] sync ={{"end0","tbn0"},
			      {"end1","tbd0"}}; 
    private String[][] eesync ={{"end0","end1","1","1"},
				{"end0","enn0","1","10"}};
    private long[] abstimes;
    private long[] reltimes; 
    private int cnt;
    private int retval = KEEPALIVE;
    private final static int BASIC_TEST = 0;
    private final static int SYNC_TEST  = 1;
    
    // define current test
    private final static int currenttest = BASIC_TEST;
  
  public void init(String args[])throws FTAArgumentException{
    msgs = args;
    cnt = 0; 
    abstimes = new long[2*sync.length];
    reltimes = new long[2*sync.length];
  }
  
  private void printActor(String msg){
    System.out.println(getName() + ": " + msg);
  } 
  
  private long diffAbsTime(int pos){
    return (abstimes[sync.length + pos] - abstimes[pos]); 
  }
  
  private long diffRelTime(int pos){
    return (reltimes[sync.length + pos] - reltimes[pos]); 
  }
 
  private void updateTimes(){
    for(int i = 0; i < sync.length; i++){
      abstimes[i] = abstimes[sync.length + i];
      reltimes[i] = reltimes[sync.length + i];
    }
  }
  
  private int cntTags(FTATagIterator it) {
    int tagcnt = 0;
    try {
      while(it.hasNext()){
	    //     printActor("tag:" + it.next()); 
	it.next();
	tagcnt++;
      }
    }
    catch(FTATagboardFlipException e) {
      tagcnt = 0;
    }
    return tagcnt;
  }

  private void initETSync() 
    throws FTACreationException,FTAUnknownObjectException{
    for(int i = 0;i < sync.length;i++){
      for(int j = 1; j < sync[i].length; j++){
	addETSync(sync[i][0],sync[i][j]);
	System.out.println("sync added: " + sync[i][0]+"-"+sync[i][j]);
      } 
    }
  }
  private void initEESync()  
    throws FTACreationException,FTAUnknownObjectException
    {
      for(int i = 0 ; i < eesync.length; i++){
	addEESync(eesync[i][0],eesync[i][1],
		  Integer.parseInt(eesync[i][2]),
		  Integer.parseInt(eesync[i][3]));
      }
    }
  
  private void printEnsembleTimes(String ensemble,int pos)
    throws FTAArgumentException,FTAUnknownObjectException{
    abstimes[sync.length + pos] 
      = getEnsembleTime(ensemble,ABSOLUTE); 
    reltimes[sync.length + pos] 
      = getEnsembleTime(ensemble,RELATIVE);
    printActor("time abs ("+ensemble+"): " + abstimes[sync.length + pos] + 
	       " (" + diffAbsTime(pos) + ")" );
    printActor("time rel ("+ensemble+"): " + reltimes[sync.length + pos] + 
	       " (" + diffRelTime(pos) + ")" );
  }
  
  private void countTags() 
    throws FTAArgumentException,FTAUnknownObjectException{	
    FTATagIterator it;
    long tt;
    for(int i = 0;i < sync.length;i++){
      // get ensemble time
      printEnsembleTimes(sync[i][0],i);
      for(int j = 1; j < sync[i].length; j++){  
	it =  getTagIterator(sync[i][j],
			     new FTATagPattern("jfta.demo.test.TestTag"));
	tt = getTagboardTime(sync[i][j],RELATIVE); 
	printActor("#tags on "+sync[i][j]+" (at time "+ tt +") " + 
		   cntTags(it));
      }
    }
    }
  private void initSync()
    throws FTACreationException,FTAUnknownObjectException{
      initETSync();
      initEESync();
  }
  
  private void addEESync(int pos) 
    throws FTACreationException,FTAUnknownObjectException{
    if(pos <= eesync.length)
      addEESync(eesync[pos][0],eesync[pos][1],
		Integer.parseInt(eesync[pos][2]),
		Integer.parseInt(eesync[pos][3]));
  }
  
  private void remEESync(int pos)	
    throws FTARemovalException,FTAUnknownObjectException{
      if(pos <= eesync.length)
	removeEESync(eesync[pos][0],eesync[pos][1]);
  }

  private void dynamicsEETest() 
    throws FTACreationException,FTARemovalException,FTAUnknownObjectException{
    int value = cnt / 10;
    int modvalue = cnt % 10;
    if(modvalue == 0)
      {
	if((value % 2) == 1)
	  remEESync(0);
	else
	  addEESync(0);
      }
  }
  
  private void addETSync(int pos)
    throws FTACreationException,FTAUnknownObjectException {
    for(int i = 1; i < sync[pos].length;i++){
      addETSync(sync[pos][0],sync[pos][i]);
    } 
  }
  
  private void remETSync(int pos)
    throws FTARemovalException,FTAUnknownObjectException {
    for(int i = 1; i < sync[pos].length;i++){
      removeETSync(sync[pos][0],sync[pos][i]);
    } 
  }
  
  private void dynamicsETTest()
    throws FTACreationException,FTARemovalException,FTAUnknownObjectException {
    int value = cnt / 10;
    int modvalue = cnt % 10;
    if(modvalue == 0)
      {
	if((value % 2) == 1)
	  remETSync(0);
	else
	  addETSync(0);
      }
  }
  
  
  public int eval() throws FTAEvalActorException{
    try 
      {
	++cnt; 
	switch(currenttest){
	case BASIC_TEST:
	    printActor("alive.");
	    if(getName().equals("actor0-0"))
		{
		    putTag("tbd0","jfta.demo.test.TestTag",msgs); 
		    //  printActor("putTag done.");
		    flipOrder("tbd0");
		    //   printActor("flipOrder done.");
		    
		    long time = getTagboardTime("tbd0",RELATIVE);
		    printActor("tbd0 current time is " + time);
		    int i = getTagCount("tbd0",new FTATagPattern("jfta.demo.test.TestTag")); 
		    time = getTagboardTime("tbd0",RELATIVE); 
		    printActor("tbd0 current time is " + time + " and " + i + " tags.");
		    FTATagIterator it = 
			getTagIterator("tbd0",
				       new FTATagPattern("jfta.demo.test.TestTag"));
		    printActor("getIterator done: " + it);
		    try{
			while(it.hasNext()){
			    printActor((it.next()).toString());
			} 
			printActor("survived while loop.");
		    }
		    catch(FTATagboardFlipException e){
			printActor("tagboard has flipped.");
		    } 
		    System.gc();
		}
	    //printActor("exit switch.");
	    break; 
	case SYNC_TEST:
	    if(getName().equals("actor0-0"))
		{
		    putTag("tbn0","jfta.demo.test.TestTag",msgs);
		    if(cnt==1) 
			initSync();
		    dynamicsEETest();
		    dynamicsETTest();
		    countTags();	 
		    updateTimes();
		}
	    putTag("tbd1","jfta.demo.test.TestTag",msgs);	
	    putTag("tbd0","jfta.demo.test.TestTag",msgs);	
	    //	flipOrder("tbd1");
	    
	    if(cnt == 1000)
		retval = FTAActor.TERMINATE;
	    updateTimes();
	    break;
	default: System.err.println("specify current test.");
	    return FTAActor.TERMINATE; 
	}
      }
    catch(Exception e) {
      System.out.println("Exception raised: " + e);
      e.printStackTrace();
      return FTAActor.TERMINATE;
    }
    if(cnt < 5)
	return retval;
    else
	return FTAActor.TERMINATE;

  }

  public TestActor(){super();}
}
