// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: /home/rteams/jfta/jfta/ensemble/FTAEnsembleMain.java

package jfta.ensemble;

import java.util.*;
import java.net.*;

import jfta.base.*;
import jfta.bootstrap.*;
import jfta.exception.*;
import jfta.system.*;
import jfta.tagboard.*;
import jfta.tags.*;
import jfta.ensemble.*;
import jfta.actors.*;
import jfta.http.*;

/**
 * EnsembleMain (EM) is a role of an ensemble. It is responsible for
 * synchronization with other ensembles or tagboards. It stores the
 * information about available Ensemblesubs on other hosts. It
 * distributes addActor requests and provides the ensemble tick.
 */
public class FTAEnsembleMain extends Object 
    implements FTAServletI,FTAEvalableI {

    private final static int EMS_INIT = 0;
    private final static int EMS_STOPPED = 1;
    private final static int EMS_OWNERWILLWAIT = 2;
    private final static int EMS_OWNERRUN = 3;
    private final static int EMS_NOOWNERRUN = 4;
    private final static int EMS_WAITING = 5;
    private final static String[] _States = {
	"INIT","STOPPED","OWNERWILLWAIT","OWNERRUN","NOOWNERRUN","WAITING"
    };

    private static FTAService _svc = FTAService.Instance();
  
    /* input queues */
    private FTAInputQueue _tbflip;
    private FTAInputQueue _etqueue;

    /* bucket for owned tagboards */
    FTAIHash _ownedtbs;
    int _waitpreflip,_gotpreflip,_sentconsent;
  
    /* Office management */
    private FTAOfficeManager _offices;

    /* system references, name, url, ... */
    private String _mylogname;
    private FTASysSub _mysyssub;

    private int _myobjnum;
    private EMListener _listener;
    private FTACommManager _comm;

    private boolean _stopflag;
    private int _state;

    /**
     * @roseuid 383EB170D851
     */
    public FTAEnsembleMain(String logname,FTASysSub syssub,int objnum,
			   boolean startnow) {
	super();
    
	/* general initialization */
	_mylogname = logname;
	_mysyssub = syssub;
	_myobjnum = objnum;

	_stopflag = true;
	changeState(EMS_STOPPED);

	_offices = new FTAOfficeManager(_mysyssub,objnum);
	_comm = _mysyssub.getCommManager();
	_listener = new EMListener(this);

	_etqueue = new FTAInputQueue();
	_tbflip = new FTAInputQueue();
    
	_ownedtbs = new FTAIHash();

	// finally notify everyone that main is operable, in order to
	// create the offices
	_comm.broadcastMsg
	    (FTACreateENDoneMsg.get(objnum,
				    _svc.getFQDNname(),
				    _mylogname,
				    _comm.getServerPort()));

	if (startnow)
	    doControl(FTASysSub.FTACTLSTART,0);
    }
  
    /**
     */
    public String toString() {
	return "EM_"+_mylogname;
    }

    /*package*/ void shutdown() {
	_listener.uninstall(_comm);
	_offices.uninstall();
	// TBD
    }

    public int eval() throws FTAEvalException {
	if (_begincycle) {
	    _begincycle = false;
	    //debug("worknow");
	    _offices.startCycle(false);
	}
	return KEEPALIVE;
    }

    public boolean isEval() {
	return true;
    }

    /****************************************************************/
    /* state interface
    /****************************************************************/
  
    final void removeETSync(int tbnum) {
	FTASyncRef ref = (FTASyncRef)_ownedtbs.get(tbnum);
	if (ref != null)
	    ref.markForRemoval();
    }
    
    /**
     * add ensemble tagboard ownership
     * @param tname name of owned tagboard
     */
    final void addETSync(int tbnum) {
	// enqueue synchronization, will be processed
	// when ensemble has finished actor loop
	_etqueue.enqueuesingle(tbnum);
    }
  
    /**
     * dequeue all addETSync messages
     */
    final boolean processETSync() {
	int tbnum;
	while ((tbnum = _etqueue.dequeue()) != -1) {
	    
	    /* is it already registered? */
	    FTASyncRef ref = (FTASyncRef)_ownedtbs.get(tbnum);
	    if (ref == null) {
		/* no, create a new sync element and add it to the
		 * 'sendcycle' list (to receive a cycle next time)
		 */
		ref = new FTASyncRef(tbnum,FTASyncRef.SENDCYCLE);
		_ownedtbs.put(tbnum,ref);
		
		if (FTADebug.ET)
		    FTADebug.log(0,_myobjnum,
				 "Added ETSync with " + tbnum);
	    }
	}

	// return true if there is any owner at all
	return _ownedtbs.size() > 0;
    }
    
    /**
     * dequeue all received flip orders
     */
    final void processFlipOrders() {
	int tbnum;
	while ((tbnum = _tbflip.dequeue()) != -1) {
	    _comm.directFlipOrder(_myobjnum,tbnum);
	}
    }

    /**
     * @roseuid 383EB181C9F0
     * send cycle messages to all tagboards that haven't received
     * them so far
     */
    final void sendCycleMsgs() {
	Enumeration it = _ownedtbs.elements();
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();

	    // go to state waitpreflip
	    if (ref.changeStateIf(FTASyncRef.SENDCYCLE,
				  FTASyncRef.WAITPREFLIP))
		_waitpreflip++;

	    // send out cycle messages
	    if (ref.isState(FTASyncRef.WAITPREFLIP))
		_comm.directCycle(_myobjnum,ref.getObjnum());
	}
    }
    
    /**
     * asynchronously receive preFlip messages
     */
    final boolean doPreFlip(int tbnum) {
	if (FTADebug.ET) {
	    FTADebug.log(0,_myobjnum,"got PREFLIP from " + tbnum);
	    //debug("preflip");
	}

	/* mark the element as 'PREFLIP received' */
	FTASyncRef sync = (FTASyncRef)_ownedtbs.get(tbnum);
	if (sync != null && 
	    sync.changeStateIf(FTASyncRef.WAITPREFLIP,
			       FTASyncRef.GOTPREFLIP)) {
	    _waitpreflip--; _gotpreflip++;
	}

	// all preflips received?
	return _waitpreflip==0;
    }
    
    /**
     * send a single consent
     */
    /*package*/ final void sendConsent(int tbnum) {
	FTASyncRef sync = (FTASyncRef)_ownedtbs.get(tbnum);
	if (sync.changeStateIf(FTASyncRef.GOTPREFLIP,
			       FTASyncRef.SENTCONSENT)) {
	    _gotpreflip--; _sentconsent++;
	    
	    // send a consent
	    _comm.directConsent(_myobjnum,tbnum);
	}
    }

    /**
     * send out consent message to every TB that has so far
     * sent its preflip. The ensemble will wait afterwards
     */
    final void sendConsentMsgs() {
	if (_gotpreflip > 0) {
	    //if (FTADebug.ET) {
	    //FTADebug.log(0,_myobjnum,"send consentmsgs");
	    //debug("consent");
	    //}
	    Enumeration it = _ownedtbs.elements();
	    while (it.hasMoreElements()) {
		FTASyncRef ref = (FTASyncRef)it.nextElement();
		sendConsent(ref.getObjnum());
	    }
	}
    }

    void doControl(int func,int code) {
	switch (func) {
	case FTASysSub.FTACTLSHUTDOWN:
	    // immediately go to shutdown of the EN, whatever happens
	    shutdown();
	    break;
	case FTASysSub.FTACTLSTART:
	    setStopFlag(false);
	    prepareRun();
	    break;
	case FTASysSub.FTACTLSTOP:
	    setStopFlag(true);
	    break;
	default:
	    FTADebug.fatal(0,_myobjnum,
			   "Default case in state EMState");
	}
    }

    /**
     * asynchronously receive postFlip messages
     */
    final boolean lastPostFlip(int tbnum) {

	if (FTADebug.ET) {
	    FTADebug.log(0,_myobjnum,"got POSTFLIP from "+tbnum);
	    //debug("postflip");
	}
	FTASyncRef sync = (FTASyncRef)_ownedtbs.get(tbnum);

	// this exists
	if (sync != null) {
	    if (sync.changeStateIf(FTASyncRef.SENTCONSENT,
				   FTASyncRef.GOTPOSTFLIP)) {
		_sentconsent--;
	    }
	}

	// all pending consents finished?
	return (_sentconsent + _gotpreflip + _waitpreflip) == 0;
    }

    final void processPostFlips() {

	/* move all postflip TBs back to cycle state */
	boolean redo=true;
	while (redo) {
	    redo = false;
	    Enumeration it = _ownedtbs.elements();
	    while (it.hasMoreElements()) {
		FTASyncRef ref = (FTASyncRef)it.nextElement();
		if (ref.changeStateIf(FTASyncRef.GOTPOSTFLIP,
				      FTASyncRef.SENDCYCLE)) {
		    if (ref.isMarkedForRemoval()) {
			_ownedtbs.remove(ref.getObjnum());
			redo = true;
			break;
		    }
		}
	    }
	}

	//if (FTADebug.ET)
	//    FTADebug.log(0,_myobjnum,"leave sync");
    }

    /***********************************************************/
    /* this adapter manages registered messages */
    /***********************************************************/
    private class EMListener extends FTAListenerAdapter {
	private FTAEnsembleMain _em;
	public EMListener(FTAEnsembleMain em) {
	    super();
	    _em = em;

	    _comm.addListener(FTAMsg.CONTROL,this);
	    _comm.addListener(FTAMsg.PREFLIP,this,_myobjnum);
	    _comm.addListener(FTAMsg.POSTFLIP,this,_myobjnum);
	    _comm.addListener(FTAMsg.ADDETSYNC,this,_myobjnum);
	    _comm.addListener(FTAMsg.REMOVEETSYNC,this,_myobjnum);
	    _comm.addListener(FTAMsg.CREATEENOFFICEDONE,this,
			      _myobjnum);
	    _comm.addListener(FTAMsg.ACKCYCLE,this,_myobjnum);
	    _comm.addListener(FTAMsg.FLIPORDERREQ,this,_myobjnum);
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.CONTROL,this);
	    comm.removeListener(FTAMsg.PREFLIP,this);
	    comm.removeListener(FTAMsg.POSTFLIP,this);
	    comm.removeListener(FTAMsg.ADDETSYNC,this);
	    comm.removeListener(FTAMsg.REMOVEETSYNC,this);
	    comm.removeListener(FTAMsg.CREATEENOFFICEDONE,this);
	    comm.removeListener(FTAMsg.ACKCYCLE,this);
	    comm.removeListener(FTAMsg.FLIPORDERREQ,this);
	}
    
	public synchronized void notifyCreateENOfficeDone(int host) {
	    // not processed by state machine
	    if (_offices.createDone(host) && !_stopflag) {
		doControl(FTASysSub.FTACTLSTART,0);
	    }
	}

	public synchronized void notifyAckCycle(int officenum,
						long cyc,boolean fin) {
	    if (FTADebug.WT)
		FTADebug.log(0,_myobjnum,"got WORKDONE " + cyc +
			     " from " + officenum + " fin="+fin);
	    if (_offices.acknowledgeDone(officenum,cyc,fin)) {
		if (_state==EMS_OWNERRUN) {
		    // perform owning post work tasks:
		    // send out cycle messages to TBs
		    sendCycleMsgs();

		    // start a new work phase, because we haven't yet seen
		    // any preflips
		    prepareRun();
		} else if (_state==EMS_NOOWNERRUN) {
		    // perform post work tasks for non-owning ensemble
		    processFlipOrders();
		    prepareRun();
		} else if (_state==EMS_OWNERWILLWAIT) {
		    // change state to WAITING
		    changeState(EMS_WAITING);
		    // send out cycle messages to TBs
		    sendCycleMsgs();
		    // send consent msgs for already seen preflips
		    sendConsentMsgs();
		}
	    }
	}

	public synchronized void notifyFlipOrderReq(int tbnum) {
	    if (FTADebug.ET)
		FTADebug.log(0,_myobjnum,
			     "got FLIPORDERREQ to " + tbnum);
	    // not processed by statemachine
	    _tbflip.enqueuesingle(tbnum);
	}	

	public synchronized void notifyAddETSync(int ennum,int tbnum) {
	    // not processed by state machine
	    addETSync(tbnum);
	}
    
	public synchronized void notifyRemoveETSync(int ennum,int tbnum) {
		removeETSync(tbnum);
	}

	// not synchronized because already sync'd via directPreflip
	public void notifyPreFlip(int tbnum) {
	    boolean allpreflips = doPreFlip(tbnum);
	    if (_state==EMS_OWNERRUN && allpreflips)
		// change to state OWNERWILLWAIT immediately
		changeState(EMS_OWNERWILLWAIT);
	    else if (_state==EMS_WAITING) {
		// got a preflip, but we are already waiting
		// so send out a consent immediately to this tb
		sendConsent(tbnum);
	    }
	    // otherwise if _state==EMS_OWNERWILLWAIT
	    // simply process preflip
	}

	// not synchronized because already sync'd via directPostflip
	public void notifyPostFlip(int tbnum) {
	    if (lastPostFlip(tbnum)) {
		// all pending postflips received
		// copy the received postflips to the sendcycle bucket
		processPostFlips();
		// and go back to run, or stop
		prepareRun();
	    }
	}

	// issued by SysMain to control operation
	public synchronized void notifyControl(int function, int code) {
	    doControl(function,code);
	}
    }

    private String printStates() {
	StringBuffer sb = new StringBuffer();
	Enumeration it = _ownedtbs.elements();
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();
	    sb.append(ref.toString()+' ');
	}
	return sb.toString();
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHttpServer server = req.getServer();
	FTADaemon daemon = (FTADaemon)server.getContainer();
	FTAHtmlFile f = new FTAHtmlFile("FTAEnsembleMain " + _mylogname);
	f.h2("Properties");
	f.beginUL();
	f.propitem("Logical name",_mylogname);
	f.propitem("Hostname",_svc.getFQDNname());
	f.propitem("ObjectNum",_myobjnum);
	f.propitem("Next Tick Cycle",_offices.getCycleTime()+"");
	Date now = new Date();
	f.propitem("Time",now.toString());
	f.beginUL();
	f.propitem("Own state",_States[_state]);
	f.propitem("Stop flag",_stopflag+"");
	f.propitem("Offices to wait",_offices.toString());
	f.propitem("Sync state",printStates());
	f.endUL();
	f.endUL();

	f.h2("Offices");
	f.text("This list may be inaccurate when the system is running.");
	f.beginUL();
	//Enumeration it = _offices.elements();
	//while (it.hasMoreElements()) {
	//FTAOfficeRef off = (FTAOfficeRef)it.nextElement();
	FTAOfficeRef[] offices = _offices.getOffices();
	for (int i=0; i<offices.length; i++) {
	    FTAOfficeRef off = offices[i];
	    if (off != null) {
		String host = (String)off.getHostname();
		String ensurl = _mysyssub.getBaseURL(host) +
		    "/$ENS$/"+_mylogname+'/';
		f.item(); f.anchor(ensurl,host);
	    }
	}
	f.endUL();

	return f.finish(getURL(4),"FTADaemon",getURL(2));
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
public String getURL(int n) {
	switch (n) {
	case 4: /* its daemon */
	    return _mysyssub.getURL(4);
	case 2: /* its main */
	    return _mysyssub.getURL(3);
	case 3: /* its parent */
	case 1: /* its syssub */
	    return _mysyssub.getURL(1);
	default: /* itself */
	    return _mysyssub.getURL(1)+"$ENM$/"+_mylogname+"/";
	}
    }

    /**
     * change the state of the statemachine
     */
    final void changeState(int s) {
	_state = s;
    }

    /**
     * mark TB for stopping next time
     */
    final void setStopFlag(boolean val) {
	_stopflag = val;
    }

    /**
     * initialize everything for a new run, owned or unowned
     * this will be called out of stop state, or when a
     * new run is to begin (after leaving sync state)
     */
    final void prepareRun() {
	// move new registered owned TBs to the list
	boolean isowned = processETSync();

	// go to the stop state?
	if (_stopflag)
	    changeState(EMS_STOPPED);
	else {
	    // if we were in OWNERWILLWAIT, we stay there,
	    // after the next round the system will stop
	    // This case is checked because it may happen
	    // that a preflip arrives just at a time the
	    // last workdone is received
	    if (_state != EMS_OWNERWILLWAIT) {
		// start the next workphase
		changeState(isowned ? EMS_OWNERRUN : EMS_NOOWNERRUN);
	    }

	    // start a new workphase
	    _begincycle = true;
	    //debug("worknow");
	    //_offices.startCycle(false);
	}
    }

    boolean _begincycle;


    /**
     */
    private synchronized void debug(String where) {
	FTADebug.log(0,_myobjnum,
		     where+
		     ": state="+_States[_state] +
		     " waitpreflip="+_waitpreflip+
		     " gotpreflip="+_gotpreflip+
		     " sentconsent"+_sentconsent);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * java-mode: t
 * font-lock-mode: t
 * End: 
 */
