// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTAException.java,v 1.2 2001/06/21 14:57:23 veit Exp $

package jfta.exception;

import jfta.base.*;

/**
 * This class is the base class of all FTA generated exceptions
 * @author Holger Veit
 * @version $Id: FTAException.java,v 1.2 2001/06/21 14:57:23 veit Exp $
 * @since JFTA1.0
 */
public class FTAException extends Exception {
    /**
     * @serial optional subcode for error
     */ 
    protected String _subcode = null;
  
    /**
     * argumentless Constructor. will call superclass and optionally 
     * pass the exception code to an error logger class
     * @roseuid 37F3478EA539
     */
    public FTAException() {
	super();
	_subcode = null;
	logError("FTAException");
    }
  
    /**
     * Constructor. Will pass its argument to its superclass and
     * will optionally also send a message to an error logger object
     * @param s error string to be displayed
     * @roseuid 37F3478F0C48
     */
    public FTAException(String s) {
	super(s);
	_subcode = null;
	logError(s);
    }

    /**
     * Constructor. Will pass its argument to its superclass and
     * will optionally also send a message to an error logger object
     * @param e nested exception
     */
    public FTAException(Exception e) {
	super();
	_subcode = null;
	logError("Nested Exception: " + e);
    }
  
    /**
     * Constructor that will pass the error code concatenated with
     * the subcode to its parent, optionally send this string to an
     * error logger.
     * Will store the subcode separately for later retrieval.
     * @param s error string
     * @param sub error subcode
     * @roseuid 37F347997560
     */
    public FTAException(String s, String sub) {
	super(s);
	_subcode = sub;
	logError(s + ", subcode=" + _subcode);
    }
  
    /**
     * returns the optional error subcode
     * @return the subcode, this may be <code>null</code>
     * @roseuid 37F347AA83C8
     */
    public String getSubcode() {
	return _subcode;
    }
	
    /**
     * logs the exception to an error logger device
     * @param s error string
     * @roseuid 37F347ABA198
     */
    private void logError(String s) {
	//System.err.println(s);
	//printStackTrace();
    }
}
