// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAFileServlet.java,v 1.3 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.exception.*;

/**
 * This class expands pure files and file trees below some
 * URL prefix. It needs the root path (absolute filename)
 * of some hierarchy which will be reachable when triggered.
 * it will deliver the requested file below this hierarchy
 * (or return an error reply
 */
public class FTAFileServlet extends FTAServlet {

    protected String fileroot;
    protected static FTAService _svc = FTAService.Instance();
    protected static Hashtable _mimetypes;

    /**
     * mandatory empty constructor
     */
    public FTAFileServlet() {
	super();
    }

    /**
     * initialize servlet with arguments
     * @param server backlink to server
     * @param argv argument vector
     * @exception FTAArgumentException raised by the derived class init
     * method, when some required argument is incorrect
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {
	if (argv.size() != 1 || server == null)
	    throw new 
		FTAArgumentException("FTAFileServlet: " +
				     "Need argument server and fileroot");

	fileroot = _svc.expandEnv((String)argv.get(0));

	if (_mimetypes == null) {
	    _mimetypes = new Hashtable();
	    String mimefile = server.getProperty("MimeTypes");
	    // default settings
	    if (mimefile == null) {
		_mimetypes.put("htm","text/html");
		_mimetypes.put("html","text/html");
		_mimetypes.put("txt","text/plain");
		_mimetypes.put("jpg","image/jpeg");
		_mimetypes.put("gif","image/gif");
		_mimetypes.put("class","application/octet-stream");
	    } else {
		try {
		    Properties prop = new Properties();
		    prop.load(new FileInputStream(mimefile));
		    Enumeration it = prop.propertyNames();
		    while (it.hasMoreElements()) {
			String key = (String)it.nextElement();
			String val = prop.getProperty(key);
			StringTokenizer st =
			    new StringTokenizer(val," \t\n");
			while (st.hasMoreTokens()) {
			    _mimetypes.put(st.nextToken(),key);
			}
		    }
		}
		catch (IOException e) {
		    throw new FTAArgumentException("FTAFileServlet: exc="+e);
		}
	    }
	}
    }

    /**
     * return an output file to the calling server
     * @param req request data
     * @return error code (no output of HTML) or 0 = OK
     */
    public FTAServletReply reply(FTAServletRequest req) {

	String tail = req.getTail();
	if (tail == null) tail = "/";
	
	// build path
	String testpath = fileroot + tail;
	File f = new File(testpath);
	//System.out.println("FTAFileServlet: requested file=" + f);

	if (f.isDirectory()) {
	    f = new File(testpath + "/index.html");
	}
	if (!f.exists()) {
	    System.out.println("does not exist");
	    return new FTAErrorReply(404);
	}

	// has a file
	String fp = f.getPath();
	int dot = fp.lastIndexOf('.');
	String type = null;
	if (dot != -1) {
	    String ext = fp.substring(dot+1).toLowerCase();
	    type = (String)_mimetypes.get(ext);
	}
	if (type == null) type = "text/plain";

	try {
	    // return generated reply
	    return new FTAServletReply(type,(int)f.length(),
				       new FileInputStream(f));
	}
	catch(IOException e) {
	    // should not happen
	    return null;
	}
    }
}
