// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAHttpLogHandler.java,v 1.3 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.io.*;
import java.util.*;

import jfta.exception.*;

public class FTAHttpLogHandler implements FTAHttpLogI {

    protected PrintStream _out;
    private final static String _myname = "FTAHttpLogHandler";

    public FTAHttpLogHandler() {
	_out = null;
    }

    /**
     * parse and load a loghandler string (typically from
     * httpd.cfg, which has as its first token the classname
     * to be loaded, and as the following a number of class
     * specific arguments
     * @param server backlink to the calling server
     * @param handler loghandler string
     */
    public static FTAHttpLogI load(FTAHttpServer server,String handler) {
	
	StringTokenizer st = new StringTokenizer(handler," \t\n");
	String clsname = st.nextToken();
	Vector args = null;
	if (st.hasMoreTokens()) {
	    args = new Vector();
	    while (st.hasMoreTokens()) {
		args.add(st.nextToken());
	    }
	}
	try {
	    FTAHttpLogI lg = byName(clsname);
	    lg.init(server,args);
	    //System.out.println("HttpLogger " + clsname + " loaded");
	    return lg;
	}
	catch(Exception e) {
	    return new FTAHttpLogHandler();
	}
    }

    /**
     * Load a loghandler by name
     * @param clsname java name of the handler, e.g. jfta.demo.mylogger
     * @return created logger instance
     * @exception Exception various exceptions while creating a new object 
     * may be raised
     */
    public static FTAHttpLogI byName(String clsname)
	throws Exception {
	    Class lgclass = Class.forName(clsname);
	    return (FTAHttpLogI)lgclass.newInstance();
    }

    /**
     * initialize the newly created logger object.
     * @param server backlink to the calling server
     * @param args argument vector
     * @exception FTAArgumentException thrown when incorrect
     * arguments have been passed for this class.
     */
    public void init(FTAHttpServer server,Vector args)
	throws FTAArgumentException {

	if (args.size() != 1) throw new
	    FTAArgumentException(_myname + ": requires logfile name");
	try {
	    String filename = (String)args.get(0);
	    _out = new PrintStream(new FileOutputStream(filename));
	}
	catch (IOException e) {
	    _out = null;
	    throw new FTAArgumentException(_myname + ": exc=" + e);
	}
    }

    /**
     * server calls this method upon finishing a request.
     * This is to log the request somewhere (or throw it away)
     * @param host originating host, this may be either a hostname
     * or a dotted quad.
     * @param user the originating user (may be null)
     * @param authuser the authenticated user (may be null)
     * @param request requested URL
     * @param status server reply status code
     * @param bytes number of bytes returned
     */
    public synchronized void log(String host,String user,String authuser,
				 String request,int status,int bytes) {
	if (_out != null) {
	    Date d = new Date();
	    _out.println(host + " " + (user==null ? "-" : user) +
			 " " + (authuser==null ? "-" : authuser) +
			 " [" + d + "] \"" + request + "\" " +
			 status + " " + bytes);
	}
    }

    /**
     * terminate logging. This is to properly close any open files
     * when the server terminates
     */
    public void close() {
	if (_out != null)
	    _out.close();
    }
}
