// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAServerLogServlet.java,v 1.5 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.exception.*;

/**
 * This servlet will log all activities of a server.
 * It is a singleton which may be bound to any URL (in servlet.cfg)
 * or elsewhere. It will react to either a name /clearlog.ftad
 * or any other name; in the latter case the log will be displayed,
 * in the former it will be cleared.
 */
public class FTAServerLogServlet extends FTAServlet {

    public final static int LOG_STD = 0;
    public final static int LOG_ERROR = 1;
    public final static int LOG_WARNING = 2;
    public final static int LOG_SUCCESS = 3;
    public final static int LOG_USER = 4;

    private static StringBuffer _sb;
    private static FTAServerLogServlet _instance;

    /**
     * log a message
     */
    public static void logMessage(StringBuffer log,int type,String msg) {

	/* XXX intercept here to redirect logging to elsewhere */

	String col;
	switch(type) {
	default: // normal
	    col = "000000"; break;
	case LOG_ERROR: 
	    col = "ff0000"; break;
	case LOG_WARNING:
	    col = "ffff00"; break;
	case LOG_SUCCESS:
	    col = "00ff00"; break;
	case LOG_USER:
	    col = "0000ff"; break;
	}
	log.append("<li><font color=\""+col+"\">"+msg+"</font>");
    }

    /**
     * log a message
     */
    public void logMessage(int type,String msg) {
	logMessage(_sb,type,msg);
    }

    /**
     * mandatory empty constructor
     */
    public FTAServerLogServlet() {
	super();
	if (_sb == null)
	    _sb = new StringBuffer();
    }

    /**
     * this returns a standard reply (text/html) describing the
     * error You have to set the error code first in the request.
     * @param req the servlet request
     * @return the error reply
     */
    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();

	if (head.equals("/clearlog.ftad")) {
	    _sb = new StringBuffer();
	}

	// else provide the server log
	String log = _sb.toString();
	FTAHtmlFile f = new FTAHtmlFile("Server Log");
	f.b("Note: this list may not be up-to-date. Use browser reload button");
	if (log.length() > 0) {
	    f.beginForm("/cgi-bin/clearlog.ftad");
	    f.submitButton("Clear Log");
	    f.endForm();
	    f.hr();
	}
	f.beginUL();
	f.text(log);
	f.endUL();
	f.hr();
	f.beginForm("/cgi-bin/clearlog.ftad");
	f.submitButton("Clear Log");
	f.endForm();
	return f.finish("/","Back to Top");
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
