// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAServletRequest.java,v 1.3 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.io.*;
import java.net.*;
import java.util.*;
import jfta.base.*;

/**
 * This is the complete set of request parameters sent to a servlet to
 * be processed.
 */
public class FTAServletRequest {
    protected FTAHttpServer _server;
    protected String _completeurl;
    protected String _head;
    protected String _tail;
    protected Hashtable _cgiargs;
    protected Hashtable _mimeheader;
    protected OutputStream _stream;
    protected int _error;
    protected Object _user;
    
    /**
     * constructor for FTAServletRequest. This is used internally
     * in the server.
     * Users should not need to create a new request.
     * @param server back reference to the FTAHttpServer object
     * @param url complete URL that triggered the request
     * @param head current component of this request
     * @param header current MIME header arguments of this request
     * @param cgiargs current arguments (get or post) if called as a CGI script
     * @param stream output stream to send the result to
     */
    public FTAServletRequest(FTAHttpServer server,
			     String url,
			     String head,
			     Hashtable header,
			     Hashtable cgiargs,
			     OutputStream stream) {
	_server = server;
	_completeurl = url;
	_head = null;
	_tail = head;
	_mimeheader = header;
	_cgiargs = cgiargs;
	_stream = stream;
	_error = 0;
	_user = null;
    }
    
    /**
     * this is an internal copy constructor, which is used by the server's
     * servlet dispatcher
     * @param req request to copy from
     * @param head current head component (used to select the servlet
     * @param rest current rest path of the request
     */
    public FTAServletRequest(FTAServletRequest req,String head,String rest) {
	_server = req._server;
	_completeurl = req._completeurl;
	_head = head;
	_tail = rest;
	_mimeheader = req._mimeheader;
	_cgiargs = req._cgiargs;
	_stream = req._stream;
	_error = req._error;
	_user = req._user;
    }	
    
    /**
     * change the current "window" of head/tail of a request
     * @param newhead new head set in this request
     * @param newtail new tail set in this request
     */
    public void modifyHeadTail(String newhead,String newtail) {
	_head = newhead;
	_tail = newtail;
    }

    /**
     * shift the head/tail part one position to the left. If tail is
     * already null, do nothing
     * @return returns the new head component
     */
    public String shift() {
	if (_tail != null) {
	    int r = _tail.indexOf('/',1);
	    _head = FTAServlet.splitHead(_tail,r);
	    _tail = FTAServlet.splitTail(_tail,r);
	}
	return _head;
    }

    /**
     * return the issuing Http server object
     * @return server object
     */
    public FTAHttpServer getServer() {
	return _server;
    }
    
    /**
     * return the complete URL of the request
     * @return URL
     */
    public String getURL() {
	return _completeurl;
    }
    
    /**
     * return the head component of the request, already processed
     * @return URL
     */
    public String getHead() {
	return _head;
    }
    
    /**
     * return the rest path not yet processed
     * @return rest path
     */
    public String getTail() {
	return _tail;
    }
    
    /**
     * return the table of MIME header lines
     * @return mime header
     */
    public Hashtable getHeader() {
	return _mimeheader;
    }
    
    /**
     * return the table of passed CGI arguments (or null)
     * @return CGI arguments
     */
    public Hashtable getArgs() {
	return _cgiargs;
    }
    
    /**
     * return the output stream
     * @return output stream
     */
    public OutputStream getRawStream() {
	return _stream;
    }
    
    /**
     * set error code, to be used by other layers
     * @param error error code
     */
    public void setErrorCode(int error) {
	_error = error;
    }
    
    /**
     * return a stored error code
     * @return error code
     */
    public int getErrorCode() {
	return _error;
    }
    
    /**
     * set user data
     * @param user user data
     */
    public void setUserData(Object user) {
	_user = user;
    }
    
    /**
     * return the stored user data
     * @return user data
     */
    public Object getUserData() {
	return _user;
    }

    /**
     * Return a String representation of the request
     * @return request
     */
    public String toString() {
	return "FTAServletRequest[url="+_completeurl+", err="+_error+"]";
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
