// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTASysTBIterator.java,v 1.2 2001/06/21 14:57:26 veit Exp $

package jfta.system;

import jfta.exception.*;
import jfta.tags.*;
import jfta.tagboard.*;
import java.util.*;

/**
  * An iterator for access to tags in the FTASysTagboard
  * @author Holger Veit
  * @version $Id: FTASysTBIterator.java,v 1.2 2001/06/21 14:57:26 veit Exp $
  * @since JFTA1.0
  */
public class FTASysTBIterator extends FTATagIterator {
  private Enumeration iter;
  private FTATag nextObj;
  private FTASysTagboard stb;

  /**
    * check whether there is another element that matches the criteria
    * @return true, if there is one, false if not
    * @roseuid 380B31AF5950
  */
  public boolean hasNext() {
    if (hasFlipped()) return false;
    return nextObj != null;
  }
  
  /**
    * return the next element
    * @return the next element that matches the criteria
    * @exception FTATagboardFlipException thrown if an asynchronous
    *   event happens (a flip of the tagboard) or the iterator is
    *   at its end.
    * @roseuid 380B31AF5951
  */
  public FTATag next() throws FTATagboardFlipException {
    if (hasFlipped())
      throw new FTATagboardFlipException();
    else
      return nextObj == null ? null : readNext();
  }

  /**
    * fill the <code>nextObj</code> buffer with the next tag
    * @return the next tag that matches the criteria
    */
  private FTATag readNext() {

    // copy last found tag to ret variable
    FTATag ret = nextObj;
    
    // find another matching tag (put into nextObj)
    while (iter.hasMoreElements()) {
      // get the next element
      nextObj = (FTATag)iter.nextElement();
      // found one, return the last matching value
      return ret;
    }
    nextObj = null;
    return ret;
  }

  /**
    * Constructor. This is used internally and cannot be used
    * to produce a valid iterator.
    * @roseuid 380B39C3B3D8
    */
  public FTASysTBIterator(FTASysTagboard ts,Enumeration it) {
    super(null);
    stb = ts;
    iter = it;

    // fill the nextObject buffer
    readNext();
  }

  /**
    * return true if the tagboard has flipped in the meantime
    * @return true: tagboard has flipped
    */
  protected boolean hasFlipped() { 
    return stb.getTime() != cycletime;
  }
}
