// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTASysTagboard.java,v 1.7 2001/06/21 14:57:26 veit Exp $

package jfta.system;

import java.io.*;
import java.net.*;
import java.util.*;

import jfta.tags.*;
import jfta.base.*;
import jfta.exception.*;
import jfta.hash.*;
import jfta.system.*;
import jfta.ensemble.*;
import jfta.tagboard.*;
import jfta.http.*;
import jfta.bootstrap.*;

/**
 * FTASysTagboard is a special tagboard for FTA information.
 * It accepts tags and getTag requests.
 * It will do a flip when instructed by a flipnow message and will
 * acknowledge successful flip with a flipdone message.
 */
public class FTASysTagboard extends Object 
    implements FTAServletI,FTATagboardI {

    public final static String hosttagclass = "jfta.tags.FTAHostTag";
    public final static String tbtagclass = "jfta.tags.FTATBTag";
    public final static String entagclass = "jfta.tags.FTAENTag";

    private final static String[] publictagtypes = {
	hosttagclass, tbtagclass, entagclass
    };
    
    public final static int HOST = 0;
    public final static int TB = 1;
    public final static int EN = 2;

    private final static FTAService _svc = FTAService.Instance();
    
    private long _cycletime;
    private long _millis;
    private Hashtable[] _tagByName;
    private FTAIHash _tagByNum;
    private String _myurl;
    
    /**************************************************************/
    /* base facilities */
    /**************************************************************/

    /**
     */
    public FTASysTagboard(String url) {
	super();
	
	_myurl = url;
	_cycletime = 0;
	_millis = 0;

	_tagByName = new Hashtable[3];
	for (int i=0; i<3; i++)
	    _tagByName[i] = new Hashtable();

	_tagByNum = new FTAIHash();
    }
    
    /**
     */
    public String getName() {
	return "$SYSINFO$";
    }
    
    /**************************************************************/
    /* API */
    /**************************************************************/

    final Hashtable getTable(int idx) {
	return _tagByName[idx];
    }


    final boolean hasTBByName(String name) {
	return _tagByName[TB].containsKey(name);
    }

    final boolean hasENByName(String name) {
	return _tagByName[EN].containsKey(name);
    }

    final boolean hasHostByName(String name) {
	return _tagByName[HOST].containsKey(name);
    }

    final int getTBByName(String name) 
	throws FTAUnknownObjectException {
	FTASysTag st = 	(FTASysTag)_tagByName[TB].get(name);
	if (st==null)
	    throw new FTAUnknownObjectException("No such tagboard: "+
						name);
	return st.getObjnum();
    }

    final int getHostByName(String name) 
	throws FTAUnknownObjectException {
	FTASysTag st = 	(FTASysTag)_tagByName[HOST].get(name);
	if (st==null)
	    throw new FTAUnknownObjectException("No such host: "+
						name);
	return st.getObjnum();
    }

    final int getENByName(String name) 
	throws FTAUnknownObjectException {
	FTASysTag st = 	(FTASysTag)_tagByName[EN].get(name);
	if (st==null)
	    throw new FTAUnknownObjectException("No such ensemble: "+
						name);
	return st.getObjnum();
    }

    /**
     * return a tag name by objnum
     * @param objnum object number
     * @return logical name
     */
    public String getTagNameById(int objnum) {
	FTASysTag st = (FTASysTag)_tagByNum.get(objnum);
	return st.getLogname();
    }

    /**
     * return Tag by object number
     * @param objnum object number
     * @return FTASysTag
     */
    public FTASysTag getTagById(int objnum) 
	throws FTAUnknownObjectException {
	FTASysTag st = (FTASysTag)_tagByNum.get(objnum);
	if (st==null)
	    throw new FTAUnknownObjectException("No such Object "
						+ objnum);
	return st;
    }

    public void removeTag(int type,int objnum) {

	FTASysTag tag = (FTASysTag)_tagByNum.remove(objnum);
	if (tag != null) _tagByName[type].remove(tag.getLogname());
	
    }

    /**************************************************************/
    /* tagboard interface */
    /**************************************************************/

    /**
     * get current time (used by SysSub on behalf of Actor)
     * @return time in cycles
     */
    public long getTime() {
	return _cycletime;
    }
  
    /**
     * get current time (used by SysSub on behalf of Actor)
     * @return time in milliseconds
     */
    public long getMillis() {
	return _millis;
    }
  
    /**
     */
    public int getTagCount(String tagtype) {
	int idx = nameToIdx(tagtype);
	return idx==-1 ? 0 : _tagByName[idx].size();
    }
  
    /**
     */
    public void putTag(FTATag tag) throws FTATagException {
	throw new FTATagException("Cannot submit tags to $SYSINFO$");
    }

    /**
     * regularly add a tag to the tagboard
     */
    public void putTag1(FTASysTag tag) {
	int type = tag.getObjType();
	Hashtable h = _tagByName[type];
	String key = tag.getLogname();
	h.remove(key);
	h.put(key,tag);

	int ikey = tag.getObjnum();
	_tagByNum.remove(ikey);
	_tagByNum.put(ikey,tag);

	flip();
    }

    private int nameToIdx(String name) {
	if (name.equals(hosttagclass)) return HOST;
	else if (name.equals(entagclass)) return EN;
	else if (name.equals(tbtagclass)) return TB;
	return -1;
    }

    /**
     * @roseuid 38B3E5B8D210
     */
    public FTATagIterator getTagIterator(String tagtype) {
	int n = nameToIdx(tagtype);
	if (n==-1)
	    return new FTANullIterator(this);
	else
	    return new FTASysTBIterator(this,_tagByName[n].elements());
    }
  
    /**
     */
    public String[] getTagTypes() {
	return publictagtypes;
    }
  
    /**************************************************************/
    /* tag and flip logic */
    /**************************************************************/
  
    /**
     * @roseuid 38B3E59F9B30
     */
    public void flip() {
	_cycletime++;
	_millis = System.currentTimeMillis();
    }

    /**************************************************************/
    /* diagnostics */
    /**************************************************************/

    public void debug() {
	System.out.println("Hosts: "+ _tagByName[HOST]);
	System.out.println("ENs: "+ _tagByName[EN]);
	System.out.println("TBs: "+ _tagByName[TB]);
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();
	if (head.equals("/") || head.equals("/index.html")) {
	    FTAHtmlFile f = new FTAHtmlFile("FTATagboard $SYSINFO$");
	    f.h2("Properties");
	    f.beginUL();
	    f.item("none");
	    f.endUL();
	    f.h2("Stored tags");
	    f.text("This list is inaccurate, and links may even fail, when the system is running.");
	    for (int i=0; i<=EN; i++) {
		String tkey = publictagtypes[i];
		f.h3(tkey);
		Hashtable tl = _tagByName[i];
		Enumeration tgi = tl.keys();
		f.beginUL();
		while (tgi.hasMoreElements()) {
		    Object tid = tgi.nextElement();
		    FTASysTag tag = (FTASysTag)tl.get(tid);
		    f.item(); f.anchor(_myurl+tkey+"/"+
				       tid+"/",
				       tid.toString());
		}
		f.endUL();
	    }
	    return FTADaemon.stdFinish(f);
	} else {
	    String h1 = head.substring(1);
	    int idx = nameToIdx(h1);
	    if (idx >= 0) {
		Hashtable th = _tagByName[idx];
		head = req.shift();
		h1 = head.substring(1);
		FTASysTag tag = (FTASysTag)th.get(h1);
		if (tag != null) {
		    req.shift();
		    return tag.reply(req);
		}
	    }
	    return FTAErrorReply.error404();
	} 
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	return _myurl;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
