// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTATagboardMain.java,v 1.19 2001/06/21 14:57:27 veit Exp $

package jfta.tagboard;

import java.util.*;
import java.net.*;

import jfta.actors.*;
import jfta.base.*;
import jfta.bootstrap.*;
import jfta.system.*;
import jfta.ensemble.*;
import jfta.exception.*;
import jfta.tags.*;
import jfta.http.*;

/**
  * Main tagboard office of distributed or not distributed
  * tagboard suboffices.
  */
public class FTATagboardMain extends Object
    implements FTAServletI,FTAEvalableI {

    private final static int TMS_INIT = 0;
    private final static int TMS_STOPPED = 1;
    private final static int TMS_UNOWNEDRUN = 2;
    private final static int TMS_UNOWNEDFLIP = 3;
    private final static int TMS_OWNEDRUN = 4;
    private final static int TMS_WAITCONSENT = 5;
    private final static int TMS_OWNEDFLIP = 6;
    private final static String[] _States = {
	"INIT","STOPPED","UNOWNEDRUN","UNOWNEDFLIP",
	"OWNEDRUN","WAITCONSENT","OWNEDFLIP"
    };

    private static FTAService _svc = FTAService.Instance();

    private String _mylogname;
    private FTASysSub _mysyssub;
    private FTAIHash _owners;
    private int _waitforcycle,_waitforconsent;
    private FTAOfficeManager _offices;
    private FTACommManager _comm;
    private int _myobjnum;
    private FTAListenerI _listener;
    private boolean _stopflag;
    private int _state;
    private boolean _begincycle;

    /***************************************************************/
    /* standard features */
    /***************************************************************/
  
    public FTATagboardMain(String logname,FTASysSub syssub,
			   int objnum,boolean startnow) {
	super();
    
	// general initialization
	_mylogname = logname;
	_mysyssub = syssub;
	_myobjnum = objnum;

	// initialize the state machine
	_stopflag = true;
	changeState(TMS_INIT);

	_comm = _mysyssub.getCommManager();
	_offices = new FTAOfficeManager(_mysyssub,objnum);
	_listener = new TMListener(this);

	_owners = new FTAIHash();
    
	// finally notify everyone that main is operable, in order to
	// create the offices
	_comm.broadcastMsg
	    (FTACreateTBDoneMsg.get(objnum,
				    _svc.getFQDNname(),
				    _mylogname,
				    _comm.getServerPort()));
	if (startnow)
	    doControl(FTASysSub.FTACTLSTART,0,true);
    }

    /**
     * do everything for shutting down:
     * send shutdown to offices
     * dissociate from callbacks
     * dissociate from syssub
     */
    void shutdown() {
	_listener.uninstall(_comm);
	_offices.uninstall();
	// TBD
    }

    /**
     */
    public String toString() {
	return "TM_"+_mylogname;
    }

    /**
     * FTAEvalable API
     */
    public int eval() throws FTAEvalException {
	_begincycle = false;
	_offices.startCycle(true);
	return KEEPALIVE;
    }

    public boolean isEval() {
	return _begincycle;
    }

    /***************************************************************/
    /* methods dealing with the various queues and lists */
    /***************************************************************/
  
    /**
     * change the state of the statemachine
     */
    final void changeState(int s) {
	_state = s;
    }

    /**
     * add a new owner to the etsync queue
     */
    final void enqueueETSync(int ennum) {
	_owners.put(ennum,new FTASyncRef(ennum,
					 FTASyncRef.ETSYNCADDED));
    }

    /**
     * remove an owner from the etsync queue
     * @return true if object successfully removed or marked for removal
     */
    final boolean unqueueETSync(int ennum,boolean stop) {
	// may be in newly added input queue (not yet processed)
	// but also may be in waitforcycle list (if this method is
	// called from STOPPED state)
	FTASyncRef ref = (FTASyncRef)_owners.get(ennum);
	if (ref != null) {
	    if (ref.isState(FTASyncRef.WAITFORCYCLE)) {
		_owners.remove(ennum);
		_waitforcycle--;
	    } else if (stop && ref.isState(FTASyncRef.ETSYNCADDED)) {
		_owners.remove(ennum);
		return true;
	    }
	}
	return false;
    }

    /**
     * remove or mark object for removal
     */
    void markETSyncForRemoval(int ennum) {
	boolean removed = unqueueETSync(ennum,false);
	if (!removed) {
	    FTASyncRef ref = (FTASyncRef)_owners.get(ennum);
	    ref.markForRemoval();
	}
    }

    /**
     * move queued etsyncs to the real waitforcycle queue
     * also add new offices
     */
    final void processETSync() {
	Enumeration it = _owners.elements();
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();
	    if (ref.changeStateIf(FTASyncRef.ETSYNCADDED,
				  FTASyncRef.WAITFORCYCLE))
		_waitforcycle++;
	}
    }

    final void sendPreflips() {
	Enumeration it = _owners.elements();
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();
	    if (ref.isState(FTASyncRef.WAITFORCONSENT))
		_comm.directPreFlip(_myobjnum,ref.getObjnum());
	}
    }

    final void sendPostflips() {
	Enumeration it = _owners.elements(); 
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();
	    if (ref.isState(FTASyncRef.WAITFORCYCLE)) {
		_comm.directPostFlip(_myobjnum,ref.getObjnum());
	    }
	}
    }

    void doControl(int func,int code,boolean stop) {
	// default behaviour
	switch (func) {
	case FTASysSub.FTACTLSHUTDOWN:
	    // immediately go to shutdown of this TB, whatever happens
	    shutdown();
	    break;
	case FTASysSub.FTACTLSTART:
	    setStopFlag(false);
	    selectNextRunState(false);
	    break;
	case FTASysSub.FTACTLSTOP:
	    if (stop)
		changeState(TMS_STOPPED);
	    else
		setStopFlag(true);
	    break;
	default:
	    FTADebug.fatal(0,_myobjnum,
			   "Default case in FTATagboardMain.doControl");
	}
    }

    /**
     * process a cycle message from an owner
     */
    final void doCycle(int owner) {
	/* pop the owner from _waitforcycle queue
	 * add it to the list of candidates to which preflip
	 * messages will be sent
	 */
	FTASyncRef ref = (FTASyncRef)_owners.get(owner);
	if (ref != null) {
	    if (ref.changeStateIf(FTASyncRef.WAITFORCYCLE,
				  FTASyncRef.WAITFORCONSENT)) {
		_waitforcycle--; _waitforconsent++;
	    }
	    
	    /* if _waitforcycle is empty go to the preflip state */
	    if (_waitforcycle == 0) {
		// from now on be in new state
		changeState(TMS_WAITCONSENT);
		// send preflips afterwards
		sendPreflips();
	    }
	}
    }

    /**
     * initiate flip of offices
     * return true if no offices existing, i.e. directly finished
     */
    final boolean initiateOfficeFlip(int nextstate) {
	// get the most recent set of offices now
	/* do we really need to? */
	if (_offices.hasOffices()) {
	    /* yes, we do */
	    // now effectively become state "flipping"
	    changeState(nextstate);

	    // send flipnow to offices
	    _begincycle = true;
	    return false;
	} else
	    return true;
    }

    /**
     * process a consent message from an owner
     */
    final void doConsent(int ennum) {
	
	/* pop owner from _waitforconsent */
	FTASyncRef ref = (FTASyncRef)_owners.get(ennum);
	if (ref != null) {
	    if (ref.changeStateIf(FTASyncRef.WAITFORCONSENT,
				  FTASyncRef.GOTCONSENT)) {
		_waitforconsent--;
	    }
    
	    /* if all consent messages received, trigger continuation */
	    if (_waitforconsent == 0) {
		// send out flipnow to all offices
		if (initiateOfficeFlip(TMS_OWNEDFLIP)) {
		    // we are already finished
		    // send postflip and select the next state to go to
		    doPostFlipAndRun();
		}
	    }
	}
    }
    
    /**
     * send postflip messages to all owners
     */
    final void doPostFlipAndRun() {

	boolean redo = true;
	while (redo) {
	    redo = false;
	    Enumeration it = _owners.elements();
	    while (it.hasMoreElements()) {
		FTASyncRef ref = (FTASyncRef)it.nextElement();
		if (ref.changeStateIf(FTASyncRef.GOTCONSENT,
				      FTASyncRef.WAITFORCYCLE)) {
		    if (ref.isMarkedForRemoval()) {
			_owners.remove(ref.getObjnum());
			redo = true;
			break;
		    } else {
			_waitforcycle++;
		    }
		}
	    }
	}		

	// select the next state to go to, and issue necessary postflips
	selectNextRunState(true);
    }

    /**
     * this will select the next run state: owned or unowned
     */
    final void selectNextRunState(boolean dopostflip) {

	// do we have to do send out postflip stuff?
	if (dopostflip) sendPostflips();

	// will we have to stop?
	if (_stopflag)
	    changeState(TMS_STOPPED);
	else {
	    // change the state, so incoming stuff will go to OWNEDRUN
	    changeState(_owners.size() > 0 ? TMS_OWNEDRUN : TMS_UNOWNEDRUN);

	    // move newly arrived etsyncs into system: now expecting 
	    // cycle messages, if appropriate
	    processETSync();
	}
    }

    /**
     * mark TB for stopping next time
     */
    final void setStopFlag(boolean val) {
	_stopflag = val;
    }

    /**
     * back channel from TBSub for this TB
     */
    private class TMListener extends FTAListenerAdapter {
	private FTATagboardMain _tm;
	public TMListener(FTATagboardMain tm) {
	    super();
	    _tm = tm;

	    _comm.addListener(FTAMsg.CONTROL,this);
	    _comm.addListener(FTAMsg.ADDETSYNC,this,_myobjnum);
	    _comm.addListener(FTAMsg.REMOVEETSYNC,this,_myobjnum);
	    _comm.addListener(FTAMsg.CREATETBOFFICEDONE,this,_myobjnum);
	    _comm.addListener(FTAMsg.CYCLE,this,_myobjnum);
	    _comm.addListener(FTAMsg.CONSENT,this,_myobjnum);
	    _comm.addListener(FTAMsg.ACKCYCLE,this,_myobjnum);
	    _comm.addListener(FTAMsg.FLIPORDER,this,_myobjnum);
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.CONTROL,this);
	    comm.removeListener(FTAMsg.ADDETSYNC,this);
	    comm.removeListener(FTAMsg.REMOVEETSYNC,this);
	    comm.removeListener(FTAMsg.CREATETBOFFICEDONE,this);
	    comm.removeListener(FTAMsg.CYCLE,this);
	    comm.removeListener(FTAMsg.CONSENT,this);
	    comm.removeListener(FTAMsg.ACKCYCLE,this);
	    comm.removeListener(FTAMsg.FLIPORDER,this);
	}

	// issued by SysMain to control operation
	public synchronized void notifyControl(int func, int code) {
	    boolean markForStop = 
		_state == TMS_UNOWNEDFLIP ||
		_state == TMS_OWNEDRUN ||
		_state == TMS_WAITCONSENT ||
		_state == TMS_OWNEDFLIP;
	    doControl(func,code,markForStop);
	}
	
	public synchronized void notifyAddETSync(int ennum,int tbnum) {
	    enqueueETSync(ennum);
	    if (_state == TMS_UNOWNEDRUN) {
		// become owned
		processETSync();
		changeState(TMS_OWNEDRUN);
	    }
	}
    
	public synchronized void notifyRemoveETSync(int ennum,int tbnum) {
	    if (_state == TMS_STOPPED)
		unqueueETSync(ennum,true);
	    else
		markETSyncForRemoval(ennum);
	}

	public synchronized void notifyCreateTBOfficeDone(int host) {
	    if (FTADebug.CR)
		FTADebug.log(0,_myobjnum,
			     "createtbdone from " + host);
	    if (_offices.createDone(host) && !_stopflag)
		doControl(FTASysSub.FTACTLSTART,0,true);
	}

	public void notifyCycle(int ennum) {
	    //if (FTADebug.ET) {
	    //FTADebug.log(0,_myobjnum,
	    //		     "got CYCLE from " + ennum);
	    //}
	    if (_owners.size() > 0 && _state==TMS_OWNEDRUN)
		doCycle(ennum);
	}
	
	public synchronized void notifyConsent(int ennum) {
	    if (FTADebug.ET)
		FTADebug.log(0,_myobjnum,"got CONSENT from " + ennum);
	    if (_state == TMS_WAITCONSENT)
		doConsent(ennum);
	}
	
	public synchronized void notifyAckCycle(int officenum,
						long cyc,boolean fin) {
	    if (FTADebug.FT)
		FTADebug.log(0,_myobjnum,"got FLIPDONE "+
			     cyc +" from " + officenum);
	    
	    if (_offices.acknowledgeDone(officenum,cyc,fin)) {
		if (_state == TMS_UNOWNEDFLIP)
		    selectNextRunState(false);
		else if (_state == TMS_OWNEDFLIP)
		    doPostFlipAndRun();
	    }
	}

	public synchronized void notifyFlipOrder(int ennum) {
	    if (FTADebug.ET)
		FTADebug.log(0,_myobjnum,
			     "got FLIPORDER from " + ennum);
	    if (_state == TMS_UNOWNEDRUN)
		initiateOfficeFlip(TMS_UNOWNEDFLIP);
	}
    }

    /**
     */
    private synchronized void debug(String where) {
	FTADebug.log(0,_myobjnum,
		     where+
		     ": state="+_state +
		     " waitforcycle="+_waitforcycle+
		     " waitforconsent="+_waitforconsent);
    }

    private String printStates() {
	StringBuffer sb = new StringBuffer();
	Enumeration it = _owners.elements();
	while (it.hasMoreElements()) {
	    FTASyncRef ref = (FTASyncRef)it.nextElement();
	    sb.append(ref.toString()+' ');
	}
	return sb.toString();
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHttpServer server = req.getServer();
	FTADaemon daemon = (FTADaemon)server.getContainer();
	FTAHtmlFile f = new FTAHtmlFile("FTATagboardMain " + _mylogname);
	f.h2("Properties");
	f.beginUL();
	f.propitem("Logical name",_mylogname);
	f.propitem("Hostname",_svc.getFQDNname());
	f.propitem("ObjectNum",_myobjnum+"");
	f.propitem("Next Flip Cycle",_offices.getCycleTime()+"");
	f.propitem("State machine",_States[_state]);
	Date now = new Date();
	f.propitem("Time",now+"");
	f.beginUL();
	f.propitem("Offices to wait",_offices.toString());
	f.propitem("Sync state",printStates());
	f.endUL();
	f.endUL();

	f.h2("Offices");
	f.text("This list may be inaccurate when the system is running.");
	f.beginUL();
	//Enumeration it = _offices.elements();
	//while (it.hasMoreElements()) {
	//FTAOfficeRef off = (FTAOfficeRef)it.nextElement();
	FTAOfficeRef[] offices = _offices.getOffices();
	for (int i=0; i<offices.length; i++) {
	    FTAOfficeRef off = offices[i];
	    if (off != null) {
		String host = off.getHostname();
		String tbsurl = _mysyssub.getBaseURL(host) +
		    "/$TBS$/"+_mylogname+'/';
		f.item(); f.anchor(tbsurl,host);
	    }
	}
	f.endUL();

	return f.finish(getURL(4),"FTADaemon",getURL(2));
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	switch (n) {
	case 4: /* its daemon */
	    return _mysyssub.getURL(4);
	case 2: /* its main */
	    return _mysyssub.getURL(3);
	case 3: /* its parent */
	case 1: /* its syssub */
	    return _mysyssub.getURL(1);
	default: /* itself */
	    return _mysyssub.getURL(1)+"$TBM$/"+_mylogname+"/";
	}
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 *
 */
